using System;

using System.Net;
using System.Net.Sockets;
using System.Text;
using Westwind.Tools;

using System.Runtime.InteropServices;

using System.Threading;


namespace Westwind.InternetTools
{
		
	/// <summary>
	/// Summary description for wwSmtp.
	/// </summary>
	[ComSourceInterfaces( typeof(IwwSmtpEvents) )]
	[ClassInterface(ClassInterfaceType.AutoDual)]
	[ProgId("DotNetCom.wwSmtp")]	
	public class wwSmtp
	{
		/// <summary>
		///  Mail Server to send message through. Should be a domain name (mail.yourserver.net) or IP Address (211.123.123.123)
		/// </summary>
		public string MailServer = "";
		

		/// <summary>
		/// Port on the mail server to send through. Defaults to port 25.
		/// </summary>
		public int ServerPort = 25;

		/// <summary>
		/// Email address or addresses of the Recipient. Comma delimit multiple addresses. To have formatted names use
		/// "Rick Strahl" &lt;rstrahl@west-wind.com&gt;
		/// </summary>
		public string Recipient = "";

		public string CC = "";
		public string BCC = "";

		/// <summary>
		/// Email address of the sender
		/// </summary>
		public string SenderEmail = "";

		/// <summary>
		/// Display name of the sender (optional)
		/// </summary>
		public string SenderName = "";

		/// <summary>
		/// Message Subject.
		/// </summary>
		public string Subject = "";

		/// <summary>
		/// The body of the message.
		/// </summary>
		public string Message = "";

		/// <summary>
		/// Username to connect to the mail server.
		/// </summary>
		public string Username = "";
		/// <summary>
		/// Password to connect to the mail server.
		/// </summary>
		public string Password = "";

		public string ContentType = "text/plain";
		public string CharacterEncoding = "8bit";

		/// <summary>
		/// Holds any error message information if Error = true
		/// </summary>
		public string ErrorMsg = "";

		/// <summary>
		/// Error flag set when a failure occurs
		/// </summary>
		public bool Error = false;

		/// <summary>
		/// Determines whether wwSMTP passes back errors as exceptions or
		/// whether it sets error properties. Right now only error properties
		/// work reliably.
		/// </summary>
		public bool HandleExceptions = true;


		/// <summary>
		/// Connection timeouts for the mail server in seconds. If this timeout is exceeded waiting for a connection
		/// or for receiving or sending data the request is aborted and fails.
		/// </summary>
		public int Timeout = 15;

		private NetworkStream oNet = null;
		private TcpClient oSmtp = null;
		private bool bESMTP = false;

		public wwSmtp()
		{
		}

		
		
		/// <summary>
		/// Fully self contained mail sending method. Sends an email message by connecting 
		/// and disconnecting from the email server.
		/// </summary>
		/// <returns>true or false</returns>
		public bool SendMail(bool AdminEmail) 
		{
			if ( !this.Connect() )
				return false;

			bool Result = this.SendMessage();
			this.Close();

			if ( !Result )
				return false;

			// *** Code inserted to exaggerate the time it takes to send email
			Thread.Sleep(5000);

			return true;
		}


		/// <summary>
		/// The parameterless version of Sendmail
		/// </summary>
		protected void SendMailRun() 
		{
			this.Error  = this.SendMail(true);
		}

		/// <summary>
		/// Fully self contained method that sends email by just sending
		/// without waiting for confirmation by starting a new thread
		/// </summary>
		/// <returns></returns>
		public void SendMailAsync() 
		{
			ThreadStart delSendMail = new ThreadStart(this.SendMailRun);
			Thread myThread = new Thread(delSendMail);
			myThread.Start();

			//			/// You can also use a delegate to do the same thing
			//			ThreadStart delSendMail = new MethodInvoker(this.SendMailRun);
			//			delSendMail.BeginInvoke(null,null);
		}


		public void SendMailAsyncWithEvents() 
		{
			ThreadStart delSendMail = new ThreadStart(this.SendMailWithEvents);
			Thread myThread = new Thread(delSendMail);
			myThread.Start();

			/// If you want to use the Thread Pool you can just use a delegate
			/// delSendMail.BeginInvoke(null,null);
			
			/// Or you can use the ThreadPool directly - without a delegate
			/// NOTE: you have to add an object parameter to SendMailRunWithEvents()
			//ThreadPool.QueueUserWorkItem( new WaitCallback(SendMailRunWithEvents) );
		}

	
		/// <summary>
		/// Message Event that is fired as an email message is sent
		/// </summary>
		public event delSendMailMessages SendMailMessages;
		public delegate void delSendMailMessages(SendMailMessageModes Mode,string Message);
			
		/// <summary>
		/// Another version that accepts an object
		/// </summary>
		public event delSendMailMessagesObject SendMailMessagesObject;
		public delegate void delSendMailMessagesObject(SendMailEventArgs e);

		/// <summary>
		/// Sends a message with events firing SendMailMessagesEvents
		/// </summary>
		public void SendMailWithEvents() 
		{
			// *** Create EventArgs to pass back to caller
			SendMailEventArgs SMArgs = new SendMailEventArgs();
			bool Result = false;

			Result = this.Connect();
			if (!Result) 
			{
				if (this.SendMailMessages != null) 
				{
					// *** Send a Failure Message
					SMArgs.Mode = SendMailMessageModes.SendMailFailed;
					SMArgs.Message = "Couldn't connect to server." + this.ErrorMsg;
					this.SendMailMessages(SMArgs.Mode,SMArgs.Message);
					this.SendMailMessagesObject(SMArgs);
				}
				return;
			}
			
			if (this.SendMailMessages != null) 
			{
				SMArgs.Message = "Starting Message Send Operation";
				this.SendMailMessages(SMArgs.Mode,SMArgs.Message);
				this.SendMailMessagesObject(SMArgs);
			}

			Result = this.SendMessage();

			this.Close();

			if ( !Result )
			{
				if (this.SendMailMessages != null) 
				{
					SMArgs.Mode = SendMailMessageModes.SendMailFailed;
					SMArgs.Message = "Message Sending failed: " + this.ErrorMsg;
					this.SendMailMessages(SMArgs.Mode,SMArgs.Message);
					this.SendMailMessagesObject(SMArgs);
				}
				return;
			}

			// *** Code inserted to exaggerate the time it takes to send email
			Thread.Sleep(5000);

			/// Check whether we have handlers
			if (this.SendMailMessages != null) 
			{
				SMArgs.Mode = SendMailMessageModes.SendMailCompleted;
				SMArgs.Message = "Message sending completed with no errors";
				this.SendMailMessages(SMArgs.Mode,SMArgs.Message);
				this.SendMailMessagesObject(SMArgs);
			}

			return;
		}

		public void RaiseSendMailMessages(SendMailMessageModes MessageMode, string Message)
		{
			if (this.SendMailMessages != null)
				this.SendMailMessages(MessageMode,Message);
		}

		#region wwSMTP Low Level Implementation
		/// <summary>
		/// Connects to the mail server.
		/// </summary>
		/// <returns>True or False</returns>
		public bool Connect() 
		{
			oSmtp = new TcpClient();

			this.bESMTP = true;

			oSmtp.SendTimeout = this.Timeout * 1000;
			oSmtp.ReceiveTimeout = this.Timeout * 1000;
			
			int ServerPort = this.ServerPort;
			string Server = this.MailServer;
			string[] Parts = Server.Split(':');
			if (Parts.Length > 1) 
			{
				Server = Parts[0];
				ServerPort = int.Parse(Parts[1]);
			}

			if (Server == null || Server == "") 
			{
				this.SetError("No Mail Server specified.");
				return false;
			}

			try 
			{
				oSmtp.Connect(Server,ServerPort);
			}
			catch(Exception ex) 
			{ 
				this.SetError(ex.Message);
				return false;
			}

			oNet = oSmtp.GetStream();

			string lcResponse = this.SendReceive("EHLO " + Environment.MachineName + "\r\n");

			// *** ESMTP will send command list - which might get sent on first
			// *** buffer or with a separate one
			if (lcResponse.IndexOf("250 OK") < 0)
				lcResponse = this.Read();

			if (this.Username != null && this.Username.Length > 0 &&
				lcResponse.ToLower().IndexOf("auth ") > -1) 
			{

				lcResponse = this.SendReceive("auth login\r\n");
				if (!this.CheckResponseCode(lcResponse,"334"))
				{
					this.CloseConnection();
					return false;
				}

				string lcB64 = Convert.ToBase64String( Encoding.GetEncoding(1252).GetBytes(this.Username) );
				lcResponse = SendReceive(lcB64 + "\r\n");
				if (!this.CheckResponseCode(lcResponse,"334") )
				{
					this.CloseConnection();
					return false;
				}

				lcB64 = Convert.ToBase64String( Encoding.GetEncoding(1252).GetBytes(this.Password) );
				lcResponse = SendReceive(lcB64 + "\r\n");
				if (!this.CheckResponseCode(lcResponse,"235"))
				{
					this.CloseConnection();
					return false;
				}
			}


			/// Username password code removed
			
			return true;
		}

		/// <summary>
		/// Low level SendMessage method. Requires that Connect() be called first to open
		/// a connection. You can call this method multiple times without reconnecting to
		/// send multiple messages.
		/// </summary>
		/// <returns>True or False</returns>
		public bool SendMessage() 
		{

			string lcResponse = "";

			lcResponse = this.SendReceive("MAIL FROM: <" + this.GetEmailFromFullAddress(SenderEmail) + ">\r\n");

			if (!this.CheckResponseCode(lcResponse,"250") )
			{
				return false;
			}


			this.SendRecipients(this.Recipient);
			this.SendRecipients(this.CC);
			this.SendRecipients(this.BCC);

			/// Code for attachments removed

			lcResponse = this.SendReceive("DATA\r\n");
			if (!this.CheckResponseCode(lcResponse,"354") )
			{
				string lcResponse2 = this.Read();
				if (!this.CheckResponseCode(lcResponse2,"354") ) 
				{
					return false;
				}
			}

			this.Send("to: " + this.Recipient + "\r\n");
			this.Send("cc: " + this.CC + "\r\n");
			

			string Email = this.SenderEmail;
			if (this.SenderName != null && this.SenderName.Length > 0) 
			{
				Email = "\"" + this.SenderName + "\" <" + Email + ">";
			}
			this.Send("from: " + Email + "\r\n");
			this.Send("subject: " + this.Subject + "\r\n");
			this.Send("x-mailer: West Wind Internet Protocols for .Net\r\n");
			this.Send("Importance: normal\r\n");
			this.Send("Mime-Version: 1.0\r\n");
			this.Send("Content-Type: " + this.ContentType + "\r\n");
			this.Send("Content-Transfer-Encoding:" + this.CharacterEncoding + "\r\n");


			this.Send("\r\n" + this.Message + "\r\n");

			lcResponse = this.SendReceive(".\r\n");
			if (!this.CheckResponseCode(lcResponse,"250") )
			{
				return false;
			}

			return true;
		}


		bool SendRecipients(string lcRecipients) 
		{
			if (lcRecipients == null ||lcRecipients.Length == 0)
				return true;

			string[] loRecips = lcRecipients.Split(',');

			for (int x = 0; x < loRecips.Length; x++) 
			{
				string lcResponse = this.SendReceive("RCPT TO: <"  + loRecips[x] + ">\r\n");
			}
			return true;
		}

		/// <summary>
		/// Strips out just the email address from a full email address that might contain a display name
		/// in the format of: "Web Monitor" <rstrahl@west-wind.com>
		/// </summary>
		/// <param name="lcFullEmail">Full email address to parse. Note currently only "<" and ">" tags are recognized as message delimiters</param>
		/// <returns>only the email address</returns>
		string GetEmailFromFullAddress(string lcFullEmail) 
		{
			if (lcFullEmail.IndexOf("<") > 0) 
			{
				int lnIndex=lcFullEmail.IndexOf("<");
				int lnIndex2 = lcFullEmail.IndexOf(">");
				string lcEmail = lcFullEmail.Substring(lnIndex+1,lnIndex2-lnIndex -1);
				return lcEmail;
			}

			return lcFullEmail;
		}

		/// <summary>
		/// Close SMTP Connection and sends QUIT command. Use after calls to SendMessage()
		/// </summary>
		void Close() 
		{
			this.SendReceive("quit\r\n");
			this.CloseConnection();
		}

		/// <summary>
		/// Closes the mail server connection. Unlike Close() this method only resets
		/// the connection objects, but doesn't send any Session exit comands.
		/// </summary>
		void CloseConnection() 
		{
			oNet.Close();
			oSmtp.Close();
		}

		/// <summary>
		/// Error setting method.
		/// </summary>
		/// <param name="lcErrormsg"></param>
		private void SetError(string lcErrorMsg)
		{
			if (lcErrorMsg == null || lcErrorMsg.Length == 0) 
			{
				this.ErrorMsg = "";
				this.Error = false;
				return;
			}

			ErrorMsg = lcErrorMsg;
			Error = true;
		}


		/// the following are worker methods for the TCP/IP handling and parsing of incoming content
		/// Note: This should eventually be extracted into a separate class.
		/// 
		void Send(string lcText) 
		{
			if (!this.HandleExceptions)
				this.WriteToStream(oNet,lcText);
			else 
			{
				try 
				{
					this.WriteToStream(oNet,lcText);
				}
				catch(Exception e) 
				{
					this.SetError(e.Message);
				}
			}
		}
		string Read() 
		{
			if (!this.HandleExceptions)
				return this.ReadFromStream(this.oNet);
			else 
			{
				try 
				{
					return this.ReadFromStream(this.oNet);
				}
				catch(Exception e) 
				{
					this.SetError(e.Message);
				}
				return "";
			}
		}

		string SendReceive(string lcText)
		{
			this.WriteToStream(this.oNet,lcText);
			return this.ReadFromStream(this.oNet);
		}
		bool CheckResponseCode(string lcResponse, string lcCode) 
		{
			int CodeLength = lcCode.Length;
			int ResponseLength = lcResponse.Length;
			if (ResponseLength < CodeLength)
				return false;

			if (lcResponse.Substring(0,lcCode.Length) != lcCode) 
			{
				this.ErrorMsg = lcResponse;
				return false;
			}

			return true;
		}
		private void WriteToStream(NetworkStream nw, string line)
		{
			try
			{
				Byte[] arrToSend = Encoding.ASCII.GetBytes(line);
				nw.Write(arrToSend, 0, arrToSend.Length);
			}
			catch(Exception e)
			{
				this.SetError(e.Message);
			}
		}

		private string ReadFromStream(NetworkStream nw)
		{
			string returnMsg;
			try
			{
				byte[] readBuffer = new byte[1024];
				int length = nw.Read(readBuffer, 0, readBuffer.Length);
				returnMsg = Encoding.ASCII.GetString(readBuffer, 0, length);
			}
			catch(Exception e)
			{
				//throw new Exception("Read from Stream threw an exception: " + e.ToString());
				this.SetError(e.ToString());
				returnMsg = "";
			}

			return returnMsg;
		}
		#endregion


	}
	
	[ClassInterface(ClassInterfaceType.AutoDual)]
	[ProgId("DotNetCom.SendMailEventArgs")]
	public class SendMailEventArgs : EventArgs 
	{
		public string Message = "";
		public SendMailMessageModes Mode = SendMailMessageModes.StatusMessage;
	}
	public enum SendMailMessageModes 
	{
		StatusMessage=4,
		SendMailCompleted=1,
		SendMailFailed=0,
		SendMailCancelled =2
	}

}
