using System;
using System.IO;
using System.Collections;

using System.DirectoryServices;

namespace Westwind.WebMonitor.Components
{
	/// <summary>
	/// This class is used to create and managed IIS virtual directories.
	/// It's used here to create the WebMonitor Service implementation.
	/// </summary>
	public class VirtualDirectory
	{
		public WebServerType ServerType = WebServerType.IIS6;

		/// <summary>
		/// The virtual directory to work with
		/// </summary>
		public string Virtual = "";

		/// <summary>
		/// Friendly (display) name of the Web Site retrieved from IIS
		/// </summary>
		public string FriendlyName = "";
		
		/// <summary>
		/// The path of the Web Site
		/// </summary>
		public string Path = "";

		/// <summary>
		/// The IIS ADSI path name on which to work. By default: IIS://LOCALHOST/W3SVC/1/ROOT
		/// which is the default Web site if not changed.
		/// </summary>
		public string IISPath = "IIS://LOCALHOST/W3SVC/1/ROOT";

		/// <summary>
		/// The Application Pool that the virtual is created in...
		/// </summary>
		public string ApplicationPool = "";

		/// <summary>
		/// Flag whether to enable NTLM  
		/// </summary>
		public bool AuthNTLM = true;
		public bool AuthAnonymous = true;
		public bool AuthBasic = true;
		public string DefaultDocuments = "default.htm,default.aspx,default.asp";
		
		/// <summary>
		/// Used for GetWebSites to retrieve sites for a given domain or IP
		/// </summary>
		public string DomainName = "localhost";
	
		public DirectoryEntry VDir = null; 
		public string ErrorMessage = "";

		public bool CreateVirtual()
		{
			this.SetError(null);

			DirectoryEntry root= new DirectoryEntry(this.IISPath);
			if (root == null) 
			{
				this.SetError("Couldn't access root node");
				return false;
			}

			try 
			{
				this.VDir = root.Children.Add(Virtual,"IISWebVirtualDir");
			} 
			catch 
			{
				try { this.VDir= new DirectoryEntry(this.IISPath + "/" + Virtual); } 
				catch {;}
			}

			if (this.VDir == null) 
			{
				this.SetError("Couldn't create virtual.");
				return false;
			}
			
			root.CommitChanges();
			VDir.CommitChanges();

			return this.SaveVirtualDirectory();
		}

		public bool SaveVirtualDirectory() 
		{
			PropertyCollection Properties = VDir.Properties;

			try 
			{
				Properties["Path"].Value = Path;
			}
			catch (Exception ex)
			{
				this.SetError("Invalid Path provided " + ex.Message);
				return false;
			}

			this.VDir.Invoke("AppCreate",true);
			if (this.FriendlyName == "")
				VDir.Properties["AppFriendlyName"].Value = Virtual;
			else
				VDir.Properties["AppFriendlyName"].Value = this.FriendlyName;

			if (this.DefaultDocuments != "") 
				VDir.Properties["DefaultDoc"].Value = this.DefaultDocuments;

			int Flags = 0;
			if (this.AuthAnonymous)
				Flags = 1;
			if (this.AuthBasic) 
				Flags = Flags + 2;
			if (this.AuthNTLM) 
				Flags = Flags + 4;
			Properties["AuthFlags"].Value = Flags;   // NTLM AuthBasic Anonymous

			VDir.CommitChanges();

			return true;

		}

		public bool GetVirtualDirectoryEntry ()
		{
			return true;
		}


		/// <summary>
		/// Returns a list of Web Sites on the local machine
		/// </summary>
		/// <returns></returns>
		public WebSiteEntry[] GetWebSites() 
		{
			string Path = "IIS://" + this.DomainName + "/W3SVC";

			DirectoryEntry root = null;
			try 
			{
				root = new DirectoryEntry(Path);
			}
			catch 
			{
				this.SetError("Couldn't access root node");
				return null;
			}
			if (root == null) 
			{
				this.SetError("Couldn't access root node");
				return null;
			}

			ArrayList al = new ArrayList(20);

			
			foreach (DirectoryEntry Entry in root.Children) 
			{
				PropertyCollection Properties = Entry.Properties;
				
				try 
				{
					WebSiteEntry Site = new WebSiteEntry();
					Site.SiteName = (string) Properties["ServerComment"].Value;
					Site.IISPath = Entry.Path;
					al.Add(Site);
				}
				catch { ; }
			}
			
			root.Close();

			WebSiteEntry[] Sites = new WebSiteEntry[al.Count];
			for ( int x=0; x < al.Count; x++)
			{
				Sites[x] = (WebSiteEntry) al[x];
			}

			return Sites;
		}

			protected void SetError(string ErrorMessage)
		{
			if (ErrorMessage == null)
				this.ErrorMessage = "";

			this.ErrorMessage = ErrorMessage;
		}

	}

	public class WebSiteEntry 
	{
		public string SiteName = "";
		public string Comment = "";
		public string IISPath = "";
	}

	public enum WebServerType 
	{
		IIS4, IIS5, IIS6
	}
}

