using System;
using System.IO;
using System.Text;

using System.Threading;
using System.Diagnostics;
using System.Xml;
using System.Runtime.InteropServices;

using Westwind.InternetTools;

namespace Westwind.WebMonitor.Components
{
	/// <summary>
	/// This class contains all the settings relevant for monitoring a site, 
	/// handling an error and notifying a recipient.
	/// 
	/// The key methods of the class are CheckSite and CheckSiteAsync which are 
	/// high level methods that perform all the tasks associated with a single site
	///  check. These methods call lower level methods like AccessSiteHttp, 
	/// SendEmail, Log which are the actual worker methods.
	/// 
	/// Async operation is supported via CheckSiteAsync and the OnMessage event 
	/// which runs on a separate newly created thread. OnMessage events are fired 
	/// as requests are started and finished with error information embedded.	
	/// </summary>
	[ClassInterface(ClassInterfaceType.AutoDual)]
	[ProgId("WebMonitor.WebMonitorSite")]
	public class WebMonitorSite : MarshalByRefObject
	{
		
		private static object SiteLock = new object();

		/// <summary>
		/// Descriptive Name of this Web Monitor Site
		/// </summary>
		public string SiteName
		{
			get {return this.cSiteName; }
			set {this.cSiteName = value; }
		}
		string cSiteName = "New Web Monitor Site";

		/// <summary>
		/// Identifier for this site in GUID string format.
		/// <seealso>Class WebMonitorSite</seealso>
		/// </summary>
		public string SiteId 
		{
			get { return this.cSiteId; }
			set { this.cSiteId = value; }
		}
		string cSiteId = "";
				
		/// <summary>
		/// The Url to hit for a given site check. Should be in format:
		/// http://site.com:port/page.ext
		/// </summary>
		public string Url 
		{
			get { return this.cUrl; }
			set { this.cUrl = value; }
		}
		string cUrl = "";

		/// <summary>
		/// Any raw POST data to send to the server
		/// </summary>
		public string PostData 
		{
			get { return this.cPostData; } 
			set { this.cPostData = value; }
		}
		string cPostData = "";

		/// <summary>
		/// The content type for the data posted.
		/// </summary>
		public string PostContentType
		{
			get { return this.cPostContentType; }
			set { this.cPostContentType = value;  }
		}
		string cPostContentType = "application/x-www-form-urlencoded";


		/// <summary>
		/// Username for authentication if required.
		/// </summary>
		public string Username 
		{
			get { return this.cUsername; }
			set { this.cUsername = value; }
		}
		string cUsername = "";

		/// <summary>
		/// HTTP Password to access this HTTP request.
		/// <seealso>Class WebMonitorSite</seealso>
		/// </summary>
		public string Password
		{
			get { return this.cPassword; }
			set { this.cPassword = value; }
		}
		string cPassword = "";

		/// <summary>
		/// String the HTTP content is searched for on a site check. If the string is 
		/// not found the request fails. If this string is empty no search matching is 
		/// performed.
		/// </summary>
		public string SearchFor
		{
			get { return this.cSearchFor; }
			set { this.cSearchFor = value; }
		}
		string cSearchFor = "";

		/// <summary>
		/// Timeout in seconds for the site to wait for completion of the request. If exceeded an alert is sent.
		/// </summary>
		public int Timeout
		{
			get { return this.nTimeout; }
			set { this.nTimeout = value; }
		}
		int nTimeout = 15;

		/// <summary>
		/// Timeout in seconds for the site to wait for completion of the request. If 
		/// exceeded an alert is sent.
		/// </summary>
		public int CheckFrequency
		{
			get { return this.nCheckFrequency; }
			set { this.nCheckFrequency = value; }
		}
		int nCheckFrequency = 600; // 10 minutes

		/// <summary>
		/// Determines whether the site is active and checked for in the 
		/// RequiresChecking() check whether this request needs to be run.
		/// <seealso>Class WebMonitorSite</seealso>
		/// </summary>
		public bool Active
		{
			get { return this.bActive; }
			set { this.bActive = value; }
		}
		bool bActive = true;
	
		/// <summary>
		/// DateTime of when the next check should occur.
		/// </summary>
		public DateTime NextCheck 
		{
			get { return this.tNextCheck; }
			set { this.tNextCheck = value; }
		}
		DateTime tNextCheck;

		/// <summary>
		/// DateTime of the last request that was run.
		/// </summary>
		public DateTime LastCheck
		{
			get { return this.tLastCheck; }
			set { this.tLastCheck = value; }
		}
		DateTime tLastCheck;

		/// <summary>
		/// Time and Date that a site check was started.
		/// </summary>
		public DateTime SiteCheckStarted 
		{
			get { return this.tSiteCheckStarted; }
			set { this.tSiteCheckStarted = value; }
		}
		DateTime tSiteCheckStarted;

		/// <summary>
		/// Email Address used to send alert messages.
		/// </summary>
		public string EmailAddress 
		{
			get { return this.cEmailAddress; }
			set { this.cEmailAddress = value; }
		}
		string cEmailAddress = "";

		/// <summary>
		/// Email address used as the return address when error emails are sent.
		/// </summary>
		public string EmailFrom
		{
			get { return this.cEmailFrom; }
			set { this.cEmailFrom = value; }
		}
		string cEmailFrom = "";

		/// <summary>
		/// The result value from an Async request check
		/// </summary>
		public bool AsyncResult 
		{
			get { return this.lAsyncResult; }
			set { this.lAsyncResult = value; }
		}
		bool lAsyncResult = false;

		/// <summary>
		/// Message assigned to email if a failure occurs. If not set defaults to the Site Url.
		/// </summary>
		public string FailureMessage 
		{
			get 
			{ 
				if (this.cFailureMessage.Length == 0)
					this.cFailureMessage = "Site " + this.cSiteName + " is down. ";

				return this.cFailureMessage; 
			}
			set { this.cFailureMessage = value; }
		}
		string cFailureMessage = "";

		/// <summary>
		/// Message assigned to email if a failed server comes back up.
		/// </summary>
		public string BackUpMessage 
		{
			get 
			{ 
				if (this.cBackUpMessage.Length == 0)
					this.cBackUpMessage = "Site " + this.cSiteName + " is back up now.";
				return this.cBackUpMessage; 
			}
			set { this.cBackUpMessage = value; }
		}
		string cBackUpMessage = "";

		/// <summary>
		/// Contains an external process to fire when the request fails.
		/// </summary>
		public string ProcessToRun 
		{
			get { return this.cProcessToRun;  }
			set { this.cProcessToRun = value; }
		}
		string cProcessToRun = "";

		/// <summary>
		/// Contains an external process to fire when the request fails.
		/// </summary>
		public string ProcessToRun2 
		{
			get { return this.cProcessToRun2;  }
			set { this.cProcessToRun2 = value; }
		}
		string cProcessToRun2 = "";

		/// <summary>
		/// Contains the HTTP response from the last request. Typically this will be 
		/// HTML.
		/// </summary>
		public string LastResponseText 
		{
			get { return this.cLastResponseText; }
			set { this.cLastResponseText = value; }
		}
		string cLastResponseText = null;

		/// <summary>
		/// General purpose field you can use to attach some information to the
		/// request. Not used internally but can be used to pass generic info to
		/// a handler
		/// </summary>
		public string Tag
		{
			get { return this.cTag; }
			set { this.cTag = value; }
		}
		string cTag;


		/// <summary>
		/// Configuration object that is used to get information for 
		/// calling the mail server and default configuration information.
		/// This property must be assigned in the constructor!
		/// </summary>
		public WebMonitorConfig Config
		{
			get { return this.oConfig; }
			set { this.oConfig = value; }
		}
		WebMonitorConfig oConfig = null;

		/// <summary>
		/// Contains the time it took to run the last HTTP request.
		/// <seealso>Class WebMonitorSite</seealso>
		/// </summary>
		public double LastRequestTime 
		{	
			get { return this.dLastRequestTime; }
		}
		double dLastRequestTime;
		
		/// <summary>
		/// Error flag that is true if an operation fails.
		/// <seealso>Class WebMonitorSite</seealso>
		/// </summary>
		public bool Error
		{
			get { return this.m_Error; }
			set { this.m_Error = value ; }
		}
		bool m_Error = false;

		/// <summary>
		/// Error Message that was set when the last request failed.
		/// <seealso>Class WebMonitorSite</seealso>
		/// </summary>
		public string ErrorMsg
		{
			get { return this.m_ErrorMsg; }
			set { this.m_ErrorMsg = value; }
		}
		string m_ErrorMsg = "";

		bool lLastRequestSuccess = true;
		int	 nFailures = 0;
		Thread oAsyncThread = null;



		/// <summary>
		/// Default constructor for this class. Pass in a WebMonitorConfig object
		/// that is required for operation of this object.
		/// </summary>
		public WebMonitorSite(WebMonitorConfig Config)
		{
			this.SetConfig(Config);
			this.SiteId = Guid.NewGuid().ToString();

		}

		/// <summary>
		/// Default Constructor. Required for Serialization to work, but you
		/// should use the alternate version that passes in the required Config 
		/// object.
		/// </summary>
		public WebMonitorSite() 
		{
			this.SiteId = Guid.NewGuid().ToString();
		}
		
		/// <summary>
		/// Assigns the Config member on the WebMonitorSite object. The config object 
		/// is used to specify default HTTP and system settings.
		/// <seealso>Class WebMonitorSite</seealso>
		/// </summary>
		/// <param name="loConfig as WebMonitorConfig"></param>
		/// <returns>Void</returns>
		public void SetConfig(WebMonitorConfig Config) 
		{
			this.oConfig = Config;
		}
				
		/// <summary>
		/// Assigns the default values that are stored in the main configuration. 
		/// This method is called when a new entry is created with default values.
		/// </summary>
		/// <returns></returns>
		public bool SetDefaultValuesFromConfig() 
		{
			if (this.oConfig == null)
				return false;

			this.cEmailFrom = this.oConfig.EmailFrom;
			this.cEmailAddress = this.oConfig.Recipients;

			this.cFailureMessage = "Site " + this.cSiteName + " is down.";
			this.cBackUpMessage = "Site " + this.cSiteName + " is back up.";
			return true;
		}


		/// <summary>
		/// Low level method that goes out and accesses a URL based on the internal 
		/// URL propoerty settings. Request checks for timeout, response code and
		/// the actual content compared to the SearchFor property value.
		/// </summary>
		/// <param name=""></param>
		/// <returns>Boolean</returns>
		public bool AccessSiteHttp()
		{
			if ( this.OnHttpRequest != null) 
			{
				if ( this.OnHttpRequest(this) )
					return true;  // event handled HTTP retrieval
			}

			this.LastResponseText = "";
			this.dLastRequestTime = 0;

			// *** Time the request in seconds
			long lnStartTicks = DateTime.Now.Ticks;

			wwHttp loHttp = new wwHttp();
			loHttp.Timeout = this.nTimeout;
			loHttp.Username = this.Username;
			loHttp.Password = this.Password;
			loHttp.ProxyAddress = this.oConfig.ProxyServer;
			loHttp.ProxyBypass = this.oConfig.ProxyByPass;
			loHttp.ProxyPassword = this.oConfig.ProxyPassword;
			loHttp.ProxyUsername = this.oConfig.ProxyUsername;

			// *** Handle Post Data if provided
			if (this.PostData != null && this.PostData != "") 
			{
				if (this.PostContentType.ToLower() == "multipart")
					loHttp.PostMode = 2;
				else 
				{
					loHttp.PostMode = 8;
					loHttp.CustomContentType = this.PostContentType;
					loHttp.AddPostKey(this.PostData);
				}
			}
			
			//loHttp.UserAgent = @"Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; .NET CLR 1.1.4322; .NET CLR 1.2.30703)";
			try 
			{
				this.LastResponseText = loHttp.GetUrl(this.cUrl);
			}
			catch(Exception ex) 
			{
				this.SetError(ex.Message); 
			}

			long lnTotalTicks = DateTime.Now.Ticks - lnStartTicks;
			this.dLastRequestTime = (double)  ( (double) lnTotalTicks / TimeSpan.TicksPerSecond );
			
			if (loHttp.Error) 
			{
				this.SetError(loHttp.ErrorMsg);
				return false;
			}

			return true;
		}

		/// <summary>
		/// High level method that performs an entire site check, notifies the
		/// recipients on errors and takes the action specified.
		/// 
		/// This method uses the settings of the object and calls the various lower
		/// level methods to perform its tasks.
		/// 
		/// <seealso>Class WebMonitorSite</seealso>
		/// </summary>
		/// <param name=""></param>
		/// <returns>Boolean</returns>
		public bool CheckSite()
		{
			this.SiteCheckStarted = DateTime.Now;
			this.Error = false;
			this.ErrorMsg = "";

			this.RaiseOnMessage("Processing Web Request",EventMessageId.StartProcessing);

			// *** Check for success
			if (this.AccessSiteHttp() && this.SearchResponseForString() )
			{
				this.RaiseOnMessage("Request completed successfully",EventMessageId.Success);

				// *** Send a back - up message if site comes back after a failure.
				if (!this.lLastRequestSuccess)  
				{
					this.SendEmail(this.BackUpMessage,this.BackUpMessage + "\r\n\r\n" + this.cUrl);
					this.LogError(this.BackUpMessage,ErrorType.SiteBackUp);
				}
				
				// *** Figure out when to fire the next request
				this.lLastRequestSuccess = true;
				this.tSiteCheckStarted = DateTime.MinValue;
				this.tLastCheck = DateTime.Now;
				this.tNextCheck = DateTime.Now.AddSeconds((double) this.nCheckFrequency);
				this.nFailures = 0;
				
				// *** NO ERRORS - just return
				return true;
			}
			
			// *** WE HAVE ERRORS

			this.RaiseOnMessage("Request failed",EventMessageId.Failure);

			// *** Handle the site error by sending email and running a process
			// *** but only if we've not done it yet
			if (this.lLastRequestSuccess)   
			{
				this.SendEmail(this.FailureMessage,this.FailureMessage + "\r\n\r\n" + this.cUrl + "\r\n" +
					this.ErrorMsg + "\r\n\r\nLast Response String (searching for " +  this.SearchFor + "):\r\n" + 
					this.LastResponseText);

				this.LogError(this.ErrorMsg,ErrorType.Failed);
			}

			this.nFailures++;

			this.RunProcess(this.nFailures);
			
			// *** Figure out when to fire next request
			this.lLastRequestSuccess = false;
			this.SiteCheckStarted = DateTime.MinValue;
			this.tNextCheck = DateTime.Now.AddSeconds((double) this.nCheckFrequency);

			// *** OK no failures
			return false;
		}

		/// <summary>
		/// This method performs the same functionality as CheckSite does, but does so 
		/// Asynchronously.
		/// It fires a new thread and then calls CheckSite from this new thread.
		/// <seealso>Class WebMonitorSite</seealso>
		/// </summary>
		/// <param name=""></param>
		/// <returns>Void</returns>
		public void CheckSiteAsync() 
		{
			// *** Assign to local var
			WebMonitorSite loSite = this;

			ThreadStart loEntryPoint = new ThreadStart(loSite.CheckSiteAsyncRun);
			
			this.oAsyncThread = new Thread(loEntryPoint);
			this.oAsyncThread.Name = this.SiteName;
			this.oAsyncThread.Start();
		}
	
		/// <summary>
		/// The thread entry point handler method - simply calls CheckSite() and sets the result
		/// </summary>
		protected void CheckSiteAsyncRun() 
		{
			this.lAsyncResult = this.CheckSite();
		}

		/// <summary>
		/// Checks the response for the expected response.
		/// </summary>
		/// <returns></returns>
		protected bool SearchResponseForString() 
		{

			if (this.OnParseResponse != null) 
			{
				if ( this.OnParseResponse(this) )
					return true; 
			}

			if (this.cSearchFor == null || 
				this.cSearchFor.Length == 0 || 
				this.LastResponseText.IndexOf(this.cSearchFor) > -1 )
				return true;

			// *** Search string wasn't found
			this.SetError( "Content search string was not found in response." ) ;

			return false;
		}	
		/// <summary>
		/// The OnMessage Event fires on various occasions during the request process. When a
		/// request is started and when it completes. The event provides references to the Site
		/// a message and a Event Id that describes which kind of event is being fired.
		/// </summary>
		public event OnMessageHandler OnMessage;
		public delegate void OnMessageHandler(WebMonitorSite Site, string Message, EventMessageId  MsgId);


		public delegate bool WebMonitorSiteEventHandler(WebMonitorSite Site);

		/// <summary>
		/// The OnHttpRequest Event fires when the site is going out to check HTTP Requests.
		/// You can return true to indicate you've handled HTTP Retrieval.
		/// </summary>
		public event WebMonitorSiteEventHandler OnHttpRequest;
		
		/// <summary>
		/// This Event fires after an HTTP has completed and when the 
		/// site is checked to see whether the request worked.
		/// </summary>
		public event WebMonitorSiteEventHandler OnParseResponse;

		/// <summary>
		/// This event fires when an email message is sent. You can return true to 
		/// indicate you've handled this event and don't want Web Monitor to send
		/// a further message.
		/// </summary>
		public event WebMonitorSiteEventHandler OnSendEmail;
		
		/// <summary>
		/// Fired when the failure process is launched
		/// </summary>
		public event WebMonitorSiteEventHandler OnRunProcess;

		/// <summary>
		/// Fires when an error is logged into the Error Log.
		/// </summary>
		public event WebMonitorSiteEventHandler OnLogError;
		
		/// <summary>
		/// Called when an entry is logged into the detail log.
		/// Note that the log could be turned off in which case
		/// this event won't fire.
		/// </summary>
		public event WebMonitorSiteEventHandler OnLogDetail;

		/// <summary>
		/// Raises a message event on this site. You can use this to fire special
		/// events to the front end layer of the application. Normally this method
		/// is called only internally.
		/// </summary>
		/// <param name="Message">
		/// The display message that is to be sent.
		/// </param>
		/// <param name="lnMsgId">
		/// Message type
		/// </param>
		/// <returns>Void</returns>
		public void RaiseOnMessage(string Message,EventMessageId lnMsgId) 
		{
			if (this.OnMessage != null) 
			{
				if (lnMsgId == EventMessageId.Failure)
					this.OnMessage(this,DateTime.Now.ToLongTimeString().ToLower() + " - " + this.ErrorMsg + " (" + this.dLastRequestTime.ToString("#,###.###") + " seconds)",lnMsgId);
				else if (lnMsgId == EventMessageId.Success)
					this.OnMessage(this,DateTime.Now.ToLongTimeString().ToLower() + " - Process Completed (" + this.dLastRequestTime.ToString("#,###.###") + " seconds)",lnMsgId); 
				else // Generic
					this.OnMessage(this,DateTime.Now.ToLongTimeString().ToLower() + " - " + Message,lnMsgId);
			}

			if (this.oConfig != null && this.oConfig.DetailLogFileSize > 0)
				this.LogDetailRequest(lnMsgId);
		}


		/// <summary>
		/// Logs every event check to file.
		/// </summary>
		public void LogDetailRequest(EventMessageId MessageType) 
		{

			this.LogDetailRequest(MessageType,null);
		}


		/// <summary>
		/// Logs every event check to file.
		/// </summary>
		public void LogDetailRequest(EventMessageId MessageType, string Message)
		{
			
			if (MessageType == EventMessageId.StartProcessing || 
				this.oConfig.DetailLogFileSize  < 1 || 
				this.oConfig.DetailLogFile.Length == 0)
				return;

			if (this.OnLogDetail != null) {
				if (this.OnLogDetail(this) )
					return; 
			}
			
			if (Message == null || Message == "") 
			{
				if (MessageType == EventMessageId.Success) 
					Message = "";
				else
					Message = this.ErrorMsg;
			}

			string EndTag = "</WebMonitorDetailLog>\r\n";
			int EndTagLength = EndTag.Length;
			long Pos = 0; // file position check

			lock(SiteLock)
			{
				FileStream loFile = null;
				try 
				{
					loFile = new FileStream(this.oConfig.DetailLogFile,FileMode.OpenOrCreate,FileAccess.Write,FileShare.Write);
					loFile.Seek(0,SeekOrigin.End);
					Pos = loFile.Position;
				}
				catch
				{
					// *** If we can't get the file open to write just exit
					return;
				}
				

				XmlTextWriter XWriter;
				XWriter = new XmlTextWriter( (Stream) loFile, Encoding.UTF8);
				XWriter.Formatting = Formatting.Indented;
				XWriter.IndentChar = ' ';
				XWriter.Indentation = 4;

		
				bool NewFile = false;
				// *** If the file is empty write an XML doc root element
				if (Pos == 0)  
				{
					XWriter.WriteStartElement("WebMonitorDetailLog");
					NewFile = true;
				}
				else 
					// *** Skip the end tag
					loFile.Seek((long) EndTagLength * -1,SeekOrigin.End);

				XWriter.WriteStartElement("event");
				XWriter.WriteAttributeString("sitename",this.SiteName);
				XWriter.WriteAttributeString("siteid",this.SiteId);
				XWriter.WriteElementString("time",XmlConvert.ToString(DateTime.Now));
				XWriter.WriteElementString("secs",XmlConvert.ToString(this.dLastRequestTime));
				XWriter.WriteElementString("type",MessageType.ToString());
				XWriter.WriteElementString("message",Message);
				XWriter.WriteEndElement();  // Event

				if (NewFile)  
				{
					XWriter.WriteEndElement();
					XWriter.WriteRaw("\r\n");
				}
				else
					XWriter.WriteRaw(EndTag);

				XWriter.Close();
				loFile.Close();
			}	

			// *** Trim the log file by cutting in half if size exceeds max
			if (Pos > this.oConfig.DetailLogFileSize) 
			{
				lock(SiteLock) 
				{
					StreamReader sr = new StreamReader(this.oConfig.DetailLogFile,Encoding.GetEncoding(1252));
					string TFile = sr.ReadToEnd();
					sr.Close();

					TFile = TFile.Substring(this.oConfig.DetailLogFileSize /2);
					TFile = TFile.Substring(TFile.IndexOf("<event"));

					StreamWriter sw = new StreamWriter(this.oConfig.DetailLogFile,false,Encoding.UTF8);
					sw.Write("<WebMonitorDetailLog>\r\n");
					sw.Write(TFile);
					sw.Close();
				}
			}
		}

		/// <summary>
		/// Writes out request errors in XML format to the application
		/// log file.
		/// </summary>
		/// <param name="Message"></param>
		public void LogError(string Message,ErrorType Type) 
		{			
			if (this.OnLogDetail != null) 
			{
				if (this.OnLogError(this) )
					return; 
			}

			if (this.oConfig.LogFile == null || this.oConfig.LogFile.Length == 0)
				return;

			string EndTag = "</WebMonitorErrorLog>\r\n";
			int EndTagLength = EndTag.Length;
			long Pos = 0;
			lock(SiteLock)
			{
					FileStream loFile = null;
					try 
					{
						loFile = new FileStream(this.oConfig.LogFile,FileMode.OpenOrCreate,FileAccess.Write,FileShare.Write);
						Pos = loFile.Seek(0,SeekOrigin.End);
					}
					catch
					{
						return;
					}
				
				XmlTextWriter XWriter;
				XWriter = new XmlTextWriter( (Stream) loFile, Encoding.UTF8);
				XWriter.Formatting = Formatting.Indented;
				XWriter.IndentChar = ' ';
				XWriter.Indentation = 4;

				bool NewFile = false;
				if (Pos == 0)  
				{
					NewFile = true;
					XWriter.WriteStartElement ("WebMonitorErrorLog");
				}
				else 
					// *** Skip the end tag
					loFile.Seek((long) EndTagLength * -1,SeekOrigin.End);

				XWriter.WriteStartElement("event");
				XWriter.WriteAttributeString("sitename",this.SiteName);
				XWriter.WriteAttributeString("siteid",this.SiteId);
				XWriter.WriteElementString("url",this.Url);
				XWriter.WriteElementString("time",XmlConvert.ToString(DateTime.Now));
				XWriter.WriteElementString("type",Type.ToString());
				XWriter.WriteElementString("message",Message);
				XWriter.WriteElementString("searchfor",this.SearchFor);
				XWriter.WriteElementString("responsetext",this.LastResponseText);
				XWriter.WriteEndElement();  // Event

				if (NewFile)  
				{
					XWriter.WriteEndElement();
					XWriter.WriteRaw("\r\n");
				}
				else
					XWriter.WriteRaw(EndTag);

				XWriter.Close();
				loFile.Close();
			}	
		}

	
		/// <summary>
		/// Handles shutting down the currently executing thread
		/// of this instance when running CheckSiteAsync.
		/// </summary>
		public void StopThread() 
		{
			this.OnMessage = null; // clear out the event handler

			if (this.oAsyncThread != null) 
			{
				this.oAsyncThread.Abort();
				this.oAsyncThread.Join(5000); // wait for max of 5 secs for completion
				this.oAsyncThread = null;
			}
		}

		/// <summary>
		/// Queries the current site and sees if it has to be run. Based on the values 
		/// contained in the NextCheck, LastCheck, CheckFrequency and Active 
		/// properties.
		/// </summary>
		/// <returns>Boolean</returns>
		public bool RequiresChecking() 
		{
			if ( !this.bActive )
				return false;

			// *** Check for timeout - if timed out just reset
			if ( this.SiteCheckStarted != DateTime.MinValue && 
				DateTime.Now > this.SiteCheckStarted.AddSeconds(this.Timeout) ) 
			{
				this.SiteCheckStarted = DateTime.MinValue;
				this.LastCheck = DateTime.Now;
				this.NextCheck = DateTime.Now.AddSeconds(this.nCheckFrequency);

				return false;
			}

			// *** already busy 
			if (this.SiteCheckStarted != DateTime.MinValue)
				return false;

			// *** Check for all empty
			if (this.NextCheck == DateTime.MinValue && 
				this.SiteCheckStarted == DateTime.MinValue) 
			{
				this.NextCheck = DateTime.Now.AddSeconds(this.nCheckFrequency);
				return false;
			}

			// *** Finally check to see that site needs checking
			if (this.NextCheck < DateTime.Now) 
				return true;

			// *** Not time yet
			return false;
		}	

		/// <summary>
		/// Sends an email using the oConfig and internal mail settings.
		/// </summary>
		/// <param name="lcHeader">Subject of the message</param>
		/// <param name="Message">Message body</param>
		/// <returns></returns>
		public bool SendEmail(string lcHeader,string Message)
		{
			if ( this.OnSendEmail != null) 
			{
				if (this.OnSendEmail(this) )
					return true;  // event handled the request
			}
			
			try 
			{
				InternetTools.wwSmtp oSmtp = new InternetTools.wwSmtp();
				oSmtp.MailServer = this.oConfig.MailServer;
				oSmtp.Username = this.oConfig.MailServerUsername;
				oSmtp.Password = this.oConfig.MailServerPassword;
				oSmtp.SenderEmail = this.EmailFrom;
				oSmtp.Recipient = this.EmailAddress.Replace("\r\n",",");
				oSmtp.Message = Message;
				oSmtp.Subject = lcHeader;
			
				if (!oSmtp.SendMail())
				{
					this.SetError("Email Error: Access to mail server failed."); // + oSmtp.ErrorMsg);
					return false;
				}
			}
			catch(Exception ex) { this.SetError("Email Error:  " + ex.Message); return false;}

			return true;
		}

		/// <summary>
		/// Executes a process when the site is down
		/// </summary>
		/// <param name="lcProcess"></param>
		/// <param name="lcParameters"></param>
		/// <returns>Boolean</returns>
		public bool RunProcess(int lnFailures)
		{
			if ( this.OnRunProcess != null) 
			{
				if (this.OnSendEmail(this) )
					return true;  // event handled the request
			}

			string lcProcess = null;

			if (lnFailures == 1)
				lcProcess = this.ProcessToRun;
			else if(lnFailures ==2)
				lcProcess = this.ProcessToRun2;
			else
				return true;

			if (lcProcess == null || lcProcess.Length == 0)
				return false;
	
			FileInfo loFile;
			try 
			{
				loFile = new FileInfo(lcProcess);
			}
			catch(Exception)
			{
				try 
				{
					System.Diagnostics.Process.Start( lcProcess );
				}
				catch(Exception) 
				{
					this.SetError("Process to execute " + lcProcess + " doesn't exist.");
					return false;
				}
				return true;
			}

			if (!loFile.Exists) 
			{
				this.SetError("Process to execute " + lcProcess + " doesn't exist.");
				return false;
			}

			try 
			{
				ProcessStartInfo loStart = new ProcessStartInfo();
				loStart.UseShellExecute = false;
				loStart.FileName = lcProcess;
				loStart.WorkingDirectory = loFile.DirectoryName;
				//loStart.Arguments = lcParameters;

				Process loProcess = Process.Start(loStart);
			}
			catch (Exception ex) { this.SetError(ex.Message); return false; }

			return true;
		}

		/// <summary>
		/// Executes a URL or EXE file. Used for 'running' any actions to take on 
		/// failure.
		/// <seealso>Class WebMonitorSite</seealso>
		/// </summary>
		/// <param name="lcUrl as String"></param>
		/// <returns>Boolean</returns>
		//		public bool RunUrl(string lcUrl) 
		//		{
		//			wwHttp.wwHttp loHttp = new wwHttp.wwHttp();
		//			loHttp.Username = this.cUsername;
		//			loHttp.Password = this.cPassword;
		//			loHttp.Timeout = this.Timeout;
		//			loHttp.GetUrl(lcUrl);
		//
		//			if (loHttp.Error) 
		//			{
		//				this.SetError("HTTP Failure: " + loHttp.ErrorMsg);
		//				return false;
		//			}
		//
		//			return true;
		//		}


		/// <summary>
		/// Sets the Error and ErrorMsg flags.
		/// </summary>
		/// <param name="lcErrormsg"></param>
		public void SetError(string lcErrorMsg)
		{
			if (lcErrorMsg == null || lcErrorMsg.Length == 0) 
			{
				this.ErrorMsg = "";
				this.Error = false;
				return;
			}

			if (ErrorMsg == null || ErrorMsg == "")
				ErrorMsg = lcErrorMsg;
			else
				ErrorMsg += "\r\n" + lcErrorMsg;

			Error = true;
		}
		/// <summary>
		/// Resets Error and ErrorMsg flags
		/// </summary>
		public void SetError()
		{
			SetError(null);
		}

		public WebMonitorSite Clone() 
		{
			return (WebMonitorSite) this.MemberwiseClone();
		}



	}	// Class Web Monitor Site

	
	/// <summary>
	/// Event Message Id that indicates what type of event the site
	/// is reporting on.
	/// 
	/// Generic 
	/// Any type of message. Currently generic messages are sent when
	/// a request is started.
	/// 
	/// Success
	/// The request was completed successfully.
	/// 
	/// Failure
	/// The Request failed.
	/// </summary>
	public enum EventMessageId 
	{
		Generic=0,
		Success=1,
		StartProcessing=2,
		Failure=4
	}

	/// <summary>
	/// Provides info on the type of failure event that is being sent.
	/// </summary>
	public enum ErrorType
	{
		Success = 0,
		Failed = 1,
		SiteBackUp = 2,
		Generic = 4
	}

}  // NameSpace WebMonitor
