using System;

using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;

namespace Westwind.InternetTools
{
	/// <summary>
	/// A free standing SMTP implementation that doesn't rely on 
	/// CDONTS or COM Interop. Minimal implementation supports
	/// only basic functionality at this time.
	/// </summary>
	public class wwSmtp
	{
		/// <summary>
		/// Mail Server to send message through. Should be a domain name 
		/// (mail.yourserver.net) or IP Address (211.123.123.123).
		/// 
		/// You can also provide a port number as part of the string which will 
		/// override the ServerPort.
		/// <seealso>Class wwSmtp</seealso>
		/// </summary>
		public string MailServer = "";

		/// <summary>
		/// Port on the mail server to send through. Defaults to port 25.
		/// </summary>
		public int ServerPort = 25;

		/// <summary>
		/// Email address or addresses of the Recipient. Comma delimit multiple addresses. To have formatted names use
		/// "Rick Strahl" &lt;rstrahl@west-wind.com&gt;
		/// </summary>
		public string Recipient = "";

		/// <summary>
		/// Carbon Copy Recipients
		/// </summary>
		public string CC = "";

		/// <summary>
		/// Blind Copy Recipients
		/// </summary>
		public string BCC = "";

		/// <summary>
		/// Email address of the sender
		/// </summary>
		public string SenderEmail = "";

		/// <summary>
		/// Display name of the sender (optional)
		/// </summary>
		public string SenderName = "";

		/// <summary>
		/// Message Subject.
		/// </summary>
		public string Subject = "";

		/// <summary>
		/// The body of the message.
		/// </summary>
		public string Message = "";

		/// <summary>
		/// Username to connect to the mail server.
		/// </summary>
		public string Username = "";
		/// <summary>
		/// Password to connect to the mail server.
		/// </summary>
		public string Password = "";

		/// <summary>
		/// The content type of the message. text/plain default or you can set to any other type like text/html
		/// </summary>
		public string ContentType = "text/plain";

		/// <summary>
		/// Character Encoding for the message.
		/// </summary>
		public string CharacterEncoding = "8bit";

		/// <summary>
		/// The character Encoding used to write the stream out to disk
		/// Defaults to the default Locale used on the server.
		/// </summary>
		public System.Text.Encoding Encoding = null;

		/// <summary>
		/// Determines whether wwSMTP passes back errors as exceptions or
		/// whether it sets error properties. Right now only error properties
		/// work reliably.
		/// </summary>
		public bool HandleExceptions = true;

		/// <summary>
		/// An Error Message if the result is negative or Error is set to true;
		/// </summary>
		public string ErrorMessage = "";

		/// <summary>
		/// Error Flag set when an error occurs.
		/// </summary>
		public bool Error = false;

		/// <summary>
		/// Connection timeouts for the mail server in seconds. If this timeout is exceeded waiting for a connection
		/// or for receiving or sending data the request is aborted and fails.
		/// </summary>
		public int Timeout = 15;

		private NetworkStream NetStream = null;
		private TcpClient Tcp = null;
		private bool bESMTP = false;

		public wwSmtp()
		{
			//
			// TODO: Add constructor logic here
			//
		}

		/// <summary>
		/// Connects to the mail server.
		/// </summary>
		/// <returns>True or False</returns>
		public bool Connect() 
		{
			Tcp = new TcpClient();

			bESMTP = true;

			Tcp.SendTimeout = this.Timeout * 1000;
			Tcp.ReceiveTimeout = this.Timeout * 1000;

			int ServerPort = this.ServerPort;
			string Server = this.MailServer;
			string[] Parts = Server.Split(':');
			if (Parts.Length > 1) 
			{
				Server = Parts[0];
				ServerPort = int.Parse(Parts[1]);
			}

			if (Server == null || Server == "") 
			{
				this.SetError("No Mail Server specified.");
				return false;
			}
			
			try 
			{
				Tcp.Connect(Server,ServerPort);
			}
			catch(Exception ex) 
			{ 
				this.SetError(ex.Message);
				return false;
			}

			NetStream = Tcp.GetStream();

			string lcResponse = this.SendReceive("EHLO " + Environment.MachineName + "\r\n");
			if (!this.CheckResponseCode(lcResponse,"220"))
			{
				lcResponse = this.SendReceive("HELO " + this.MailServer + "\r\n");
				if (!this.CheckResponseCode(lcResponse,"220") )
				{
					this.CloseConnection();
					return false;
				}
				this.bESMTP = false;
			}
			else 
			{
				// *** ESMTP will send command list - which might get sent on first
				// *** buffer or with a separate one
				if (lcResponse.IndexOf("250 OK") < 0) 
				{
					lcResponse = this.Read();
				}
			}

			// *** Handle Login if provided
			if (this.bESMTP && this.Username != null && this.Username.Length > 0 &&
				lcResponse.ToLower().IndexOf("auth ") > -1) 
			{

				lcResponse = this.SendReceive("auth login\r\n");
				if (!this.CheckResponseCode(lcResponse,"334"))
				{
					this.CloseConnection();
					this.ErrorMessage = lcResponse;
					return false;
				}

				string lcB64 = Convert.ToBase64String( Encoding.GetEncoding(1252).GetBytes(this.Username) );
				lcResponse = SendReceive(lcB64 + "\r\n");
				if (!this.CheckResponseCode(lcResponse,"334") )
				{
					this.CloseConnection();
					return false;
				}

				lcB64 = Convert.ToBase64String( Encoding.GetEncoding(1252).GetBytes(this.Password) );
				lcResponse = SendReceive(lcB64 + "\r\n");
				if (!this.CheckResponseCode(lcResponse,"235"))
				{
					this.CloseConnection();
					return false;
				}
			}

			return true;
		}
		
		/// <summary>
		/// Fully self contained mail sending method. Sends an email message by connecting 
		/// and disconnecting from the email server.
		/// </summary>
		/// <returns>true or false</returns>
		public bool SendMail() 
		{
			if ( !this.Connect() )
				return false;

			if ( !this.SendMessage() )
				return false;

			this.Close();
			return true;
		}

		/// <summary>
		/// Fully self contained method that sends email by just sending
		/// without waiting for confirmation by starting a new thread
		/// </summary>
		/// <returns></returns>
		public void SendMailAsync() 
		{
			ThreadStart oDelegate = new ThreadStart(this.SendMailRun);
//			Thread myThread = new Thread(oDelegate);
//			myThread.Start();

			/// If you want to use the Thread Pool you can just use a delegate
			/// This will often be faster and more efficient especially for quick operations
			oDelegate.BeginInvoke(null,null);
		}

		protected void SendMailRun() 
		{
			// Create an extra reference to insure GC doesn't collect
			// the reference from the caller
			wwSmtp Email = this;  
			this.SendMail();
		}

		/// <summary>
		/// Low level SendMessage method. Requires that Connect() be called first to open
		/// a connection. You can call this method multiple times without reconnecting to
		/// send multiple messages.
		/// </summary>
		/// <returns>True or False</returns>
		public bool SendMessage() 
		{

			string lcResponse = "";

//			if (this.SenderEmail.IndexOf("<") > 0) 
//			{
//				int lnIndex=this.SenderEmail.IndexOf("<");
//				int lnIndex2 = this.SenderEmail.IndexOf(">");
//				string lcEmail = this.SenderEmail.Substring(lnIndex+1,lnIndex2-lnIndex -1);
//
//				lcResponse  = this.SendReceive("MAIL FROM: <" + lcEmail + ">\r\n");
//			}
//			else
//				lcResponse = this.SendReceive("MAIL FROM: <" + this.SenderEmail + ">\r\n");
//
			lcResponse = this.SendReceive("MAIL FROM: <" + this.GetEmailFromFullAddress(SenderEmail) + ">\r\n");

			if (!this.CheckResponseCode(lcResponse,"250") )
			{
				return false;
			}

//			lcResponse = this.SendReceive("rcpt to: <" + this.Recipient + ">\r\n");
//			if (!this.CheckResponseCode(lcResponse,"250") )
//			{
//				return false;
//			}

			this.SendRecipients(this.Recipient);
			this.SendRecipients(this.CC);
			this.SendRecipients(this.BCC);


			lcResponse = this.SendReceive("DATA\r\n");
			if (!this.CheckResponseCode(lcResponse,"354") )
			{
				lcResponse = this.Read();
				if (!this.CheckResponseCode(lcResponse,"354") ) 
				{
					return false;
				}
			}

			this.Send("to: " + this.Recipient + "\r\n");
			this.Send("cc: " + this.CC + "\r\n");
			

			string Email = this.SenderEmail;
			if (this.SenderName != null && this.SenderName.Length > 0) 
			{
				Email = "\"" + this.SenderName + "\" <" + Email + ">";
			}
			this.Send("from: " + Email + "\r\n");
			this.Send("subject: " + this.Subject + "\r\n");
			this.Send("x-mailer: wwSmtp .Net\r\n");
			this.Send("Importance: normal\r\n");
			this.Send("Mime-Version: 1.0\r\n");
			this.Send("Content-Type: " + this.ContentType + "\r\n");
			this.Send("Content-Transfer-Encoding:" + this.CharacterEncoding + "\r\n");


			this.Send("\r\n" + this.Message + "\r\n");

			lcResponse = this.SendReceive(".\r\n");
			if (!this.CheckResponseCode(lcResponse,"250") )
			{
				return false;
			}

			return true;
		}


		bool SendRecipients(string lcRecipients) 
		{
			if (lcRecipients == null ||lcRecipients.Length == 0)
				return true;

			string[] loRecips = lcRecipients.Split(',');

			for (int x = 0; x < loRecips.Length; x++) 
			{
				string lcResponse = this.SendReceive("RCPT TO: <"  + loRecips[x] + ">\r\n");
			}
			return true;
		}

		/// <summary>
		/// Strips out just the email address from a full email address that might contain a display name
		/// in the format of: "Web Monitor" <rstrahl@west-wind.com>
		/// </summary>
		/// <param name="lcFullEmail">Full email address to parse. Note currently only "<" and ">" tags are recognized as message delimiters</param>
		/// <returns>only the email address</returns>
		string GetEmailFromFullAddress(string lcFullEmail) 
		{
			if (lcFullEmail.IndexOf("<") > 0) 
			{
				int lnIndex=lcFullEmail.IndexOf("<");
				int lnIndex2 = lcFullEmail.IndexOf(">");
				string lcEmail = lcFullEmail.Substring(lnIndex+1,lnIndex2-lnIndex -1);
				return lcEmail;
			}

			return lcFullEmail;
		}

		/// <summary>
		/// Close SMTP Connection and sends QUIT command. Use after calls to SendMessage()
		/// </summary>
		void Close() 
		{
			this.SendReceive("quit\r\n");
			this.CloseConnection();
		}

		/// <summary>
		/// Closes the mail server connection. Unlike Close() this method only resets
		/// the connection objects, but doesn't send any Session exit comands.
		/// </summary>
		void CloseConnection() 
		{
			NetStream.Close();
			Tcp.Close();


		}

		/// the following are worker methods for the TCP/IP handling and parsing of incoming content
		/// Note: This should eventually be extracted into a separate class.
		/// 
		void Send(string lcText) 
		{
			if (!this.HandleExceptions)
				this.WriteToStream(NetStream,lcText);
			else 
			{
				try 
				{
					this.WriteToStream(NetStream,lcText);
				}
				catch(Exception e) 
				{
					this.SetError(e.Message);
				}
			}
		}
		string Read() 
		{
			if (!this.HandleExceptions)
				return this.ReadFromStream(this.NetStream);
			else 
			{
				try 
				{
					return this.ReadFromStream(this.NetStream);
				}
				catch(Exception e) 
				{
					this.SetError(e.Message);
				}
				return "";
			}
		}

		string SendReceive(string lcText)
		{
			this.WriteToStream(this.NetStream,lcText);
			return this.ReadFromStream(this.NetStream);
		}
		bool CheckResponseCode(string lcResponse, string lcCode) 
		{
			int CodeLength = lcCode.Length;
			int ResponseLength = lcResponse.Length;
			if (ResponseLength < CodeLength)
				return false;

			if (lcResponse.Substring(0,lcCode.Length) != lcCode) 
			{
				this.ErrorMessage = lcResponse;
				return false;
			}

			return true;
		}
		private void WriteToStream(NetworkStream nw, string line)
		{
			
			try
			{
				Byte[] arrToSend = null;
				if (this.Encoding == null)
					arrToSend = Encoding.Default.GetBytes(line);
				else
					arrToSend = this.Encoding.GetBytes(line);

				nw.Write(arrToSend, 0, arrToSend.Length);
			}
			catch(Exception e)
			{
				this.SetError(e.Message);
			}
		}

		private string ReadFromStream(NetworkStream nw)
		{
			string returnMsg;
			try
			{
				byte[] readBuffer = new byte[1024];
				int length = nw.Read(readBuffer, 0, readBuffer.Length);
				returnMsg = Encoding.Default.GetString(readBuffer, 0, length);
			}
			catch(Exception e)
			{
				//throw new Exception("Read from Stream threw an exception: " + e.ToString());
				this.SetError(e.ToString());
				returnMsg = "";
			}

			return returnMsg;
		}


		
		/// <summary>
		/// Error setting method.
		/// </summary>
		/// <param name="lcErrorMessage"></param>
		private void SetError(string lcErrorMessage)
		{
			if (lcErrorMessage == null || lcErrorMessage.Length == 0) 
			{
				this.ErrorMessage = "";
				this.Error = false;
				return;
			}

			ErrorMessage = lcErrorMessage;
			Error = true;
		}

	}
}
