using System;
using System.Runtime.InteropServices;
using System.Collections;
using System.Collections.Specialized;

using System.IO;
using System.Text;
using System.Reflection;
using System.Xml;

using System.Runtime.Remoting;

using Westwind.wwReflection.Tools;

namespace Westwind.wwReflection
{
	/// <summary>
	/// Summary description for DotNetClassInfo.
	/// </summary>
	[ClassInterface(ClassInterfaceType.AutoDual)]
	[ProgId("wwReflection.TypeParser")]
	//[Serializable]
	public class TypeParser : MarshalByRefObject
	{
		public TypeParser()
		{
		}

		/// <summary>
		/// Array of parsed objects after GetAllObjects() is called
		/// </summary>
		public DotnetObject[] aObjects;
		
		/// <summary>
		/// Array of native Type references
		/// </summary>
		public Type[] aXObjects;

		/// <summary>
		/// Number of objects retrieved
		/// </summary>
		public int nObjectCount = 0;

		/// <summary>
		/// Language Syntax to document in
		/// </summary>
		public string cSyntax = "VB";

		/// <summary>
		/// Word Wrapping options for Body and Remark text
		/// </summary>
		public bool bWordWrap = true;
 
		/// <summary>
		/// If calling GetObject() this returns a single object to parse
		/// </summary>
		public DotnetObject oObject;

		/// <summary>
		/// Constants are not used at this point
		/// </summary>
		public string[] aConstants;
		public int nConstantCount = 0;

		/// <summary>
		/// Error flag - can be checked after method calls
		/// </summary>
		public bool lError = false;

		/// <summary>
		/// Error Message if an error occurred. 
		/// </summary>
		public string cErrorMsg = "";


		/// <summary>
		/// File path to the Assembly to parse
		/// </summary>
		public string cFilename = "";

		/// <summary>
		/// Determines whether inherited members are retrieved
		/// </summary>
		public bool RetrieveDeclaredMembersOnly = true;


		/// <summary>
		/// File path to the optional Xml documentation file. 
		/// If empty XML documentation is not used.
		/// </summary>
		public string cXmlFilename = "";

	
		public string GetVersionInfo() 
		{	
			return Environment.Version.ToString() + "\r\n" + Assembly.GetExecutingAssembly().CodeBase + "\r\n" + 
				"ApplicationBase: " + AppDomain.CurrentDomain.SetupInformation.ApplicationBase + "\r\n" +
				"PrivateBinPath: " + AppDomain.CurrentDomain.SetupInformation.PrivateBinPath + "\r\n" +
				"PrivateBinProbe: " + AppDomain.CurrentDomain.SetupInformation.PrivateBinPathProbe  + "\r\n" +
				Directory.GetCurrentDirectory() + "\r\n" + AppDomain.CurrentDomain.FriendlyName + "\r\n";
		}
		/// <summary>
		/// Gets all objects into the aObjects array and parses all method/properties into the subarrays.
		/// </summary>
		/// <param name="llDontParseMethods">if true methods and properties are not parsed</param>
		/// <returns></returns>
		public int GetAllObjects(bool llDontParseMethods) 
		{
			Assembly loAssembly;
			try 
			{
				loAssembly = Assembly.LoadFrom(this.cFilename);
			}
			catch(Exception ex)
			{
				this.lError = true;
				this.cErrorMsg = ex.Message;
				return 0;
			}

			this.aXObjects = loAssembly.GetTypes();
			int lnCount = this.aXObjects.Length;
			if (lnCount == 0)
				return 0;

			this.nObjectCount = lnCount;
			this.aObjects = new DotnetObject[lnCount];


			for (int x=0; x< this.aXObjects.Length; x++) 
			{
				Type loType = this.aXObjects[x];
				this.aObjects[x] = new DotnetObject();

				this.ParseObject(this.aObjects[x],loType,llDontParseMethods);
			}

			if (this.cXmlFilename.Length > 0)  
			{
				this.ParseXmlProperties(null);
			}

			return lnCount;
		}

		/// <summary>
		/// Parses an object by name and sets the oObject 
		/// member with the result.
		/// </summary>
		/// <param name="lcClass">Class name to search</param>
		/// <returns></returns>
		public bool GetObject(string lcClass) 
		{
			if (this.aObjects == null)
				this.GetAllObjects(true);

			for (int x=0; x< this.aXObjects.Length; x++) 
			{
				Type loType = this.aXObjects[x];
				if (loType.Name.ToLower() != lcClass.ToLower()) 
					continue;

				this.oObject = new DotnetObject();
				this.oObject.cName = loType.Name;
				this.oObject.cSyntax = this.cSyntax;
				this.oObject.RetrieveDeclaredMembersOnly = this.RetrieveDeclaredMembersOnly;

				this.ParseObject(this.oObject,loType,false);

					
				//				if (loType.BaseType != null)
				//					this.oObject.cInheritsFrom = loType.BaseType.Name;
				//
				//				this.oObject.LoadMethods(loType);
				//				this.oObject.LoadProperties(loType);
				//				this.oObject.LoadEvents(loType);
				//				break;
			}

			if (this.cXmlFilename.Length > 0)  
				this.ParseXmlProperties(this.oObject);

			return true;
		}

		/// <summary>
		/// Internal method that's used to parse the currently selected object
		/// parsing methods and properties and the various object properties including
		/// object inheritance etc.
		/// </summary>
		/// <param name="loObject"></param>
		/// <param name="loType"></param>
		/// <param name="DontParseMethods"></param>
		protected void ParseObject(DotnetObject loObject, Type loType,bool DontParseMethods)
		{
			// *** Pass down high level options
			loObject.cName = loType.Name;
			loObject.cSyntax = this.cSyntax;
			loObject.RetrieveDeclaredMembersOnly = this.RetrieveDeclaredMembersOnly;

			// *** Figure out the base type and build inheritance tree

			// *** Parse basic type features
			if (loType.IsPublic || loType.IsNestedPublic)
				loObject.cScope = "public";
			else if (loType.IsNotPublic || loType.IsNestedPrivate) 
				loObject.cScope = "private";

			if ( this.cSyntax.StartsWith("VB") ) 
				loObject.cScope = wwUtils.ProperCase(loObject.cScope);

			if (loType.IsSealed) 
				if (this.cSyntax.StartsWith("VB")) 
					loObject.cOther = "NotInheritable";
				else
					loObject.cOther = "sealed";

			if (loType.IsAbstract) 
			{
				if (this.cSyntax.StartsWith("VB")) 
					loObject.cOther = "MustInherit";
				else
					loObject.cOther += "abstract";
			}

			loObject.cSignature = loType.FullName;

			loObject.cNamespace = loType.Namespace;
			loObject.cAssembly = loType.Assembly.CodeBase;
			loObject.cAssembly = loObject.cAssembly.Substring(loObject.cAssembly.LastIndexOf("/")+1);

			loObject.cType = "class";
			if (loType.IsInterface) 
				loObject.cType = "interface";
			else if (loType.IsEnum ) 
				loObject.cType = "enum";
			else if (loType.IsValueType)
				loObject.cType = "struct";
			
			// *** for this class
			if (loType.BaseType != null) 
			{
				loObject.cInheritsFrom = loObject.TypeNameForLanguage(loType.BaseType.Name);
				

				if (loObject.cInheritsFrom == "MulticastDelegate" || loObject.cInheritsFrom == "Delegate" )
					loObject.cType = "delegate";

				Type[] Implementations = loType.GetInterfaces();

				if (Implementations!=null)
				{
					foreach(Type Implementation in Implementations) 
					{
						// *** This will work AS LONG AS THE INTERFACE HAS AT LEAST ONE MEMBER!
						// *** This will give not show an 'empty' placeholder interface
						// *** Can't figure out a better way to do this...
						InterfaceMapping im = loType.GetInterfaceMap(Implementation);
						if (im.TargetMethods.Length >0 && im.TargetMethods[0].DeclaringType == loType)
							loObject.cImplements += Implementation.Name + ",";
					}
					if (loObject.cImplements != "")
						loObject.cImplements = loObject.cImplements.Substring(0,loObject.cImplements.Length-1);
				}

				ArrayList Tree = new ArrayList();
				Type Current = loType;
				while (true) 
				{
					if (Current == null)
						break;

					Tree.Add(Current.FullName);
					Current = Current.BaseType;
				}
				// *** Walk our list backwards to build the string
				for (int z = Tree.Count-1; z >= 0; z--)  
				{
					string prefix = "";
					for (int zz = 0; zz < Tree.Count - z-1; zz++) 
						prefix += "&nbsp;&nbsp;";

					if (z==0)
						loObject.cInheritanceTree +=  prefix + "<b>" + (string) Tree[z] + "</b><br>\r\n" ;
					else
						loObject.cInheritanceTree +=  prefix + (string) Tree[z] + "<br>\r\n" ;
				}
			}

			if (!DontParseMethods) 
			{
				loObject.LoadMethods(loType);
				loObject.LoadProperties(loType);
				loObject.LoadEvents(loType);
			}
		}


		/// <summary>
		/// Retrieves all objects into the aObjects array. Also
		/// parses all the subojects.
		/// </summary>
		/// <returns>Count of objects</returns>
		public int GetAllObjects() 
		{
			return this.GetAllObjects(false);
		}
		
		/// <summary>
		/// This method parses comments from C# XML documentation into the appropriate
		/// cHelpText properties.
		/// </summary>
		/// <returns></returns>
		public bool ParseXmlProperties(DotnetObject loPassedObject) 
		{
			string lcXPath;
			DotnetObject loObject;

			XmlDocument oDom = new XmlDocument();

			try 
			{
				oDom.Load("file://" + this.cXmlFilename);      
			}
			catch(Exception) 
			{
				this.cErrorMsg = "Unable to load XML Documentation file.";
				return false;
			}
			
			// *** Get object descriptions
			for (int lnObjs = 0; lnObjs < this.nObjectCount; lnObjs++) 
			{
				XmlNode loMatch;
				XmlNode loSubMatch;

				if (loPassedObject == null)
					loObject = this.aObjects[lnObjs];
				else 
				{
					loObject = loPassedObject;
					lnObjs = this.nObjectCount - 1;
				}

				if (loPassedObject != null && loPassedObject.cName.ToLower() != loObject.cName.ToLower())
					continue;

				try 
				{
					loMatch = oDom.SelectSingleNode("/doc/members/member[@name='" +
						"T:" + loObject.cNamespace + "." + loObject.cName + "']");

					loObject.cHelpText = this.GetNodeValue(loMatch,"summary");

					loObject.cRemarks = this.GetNodeValue(loMatch,"remarks");
					loObject.cExample = this.GetNodeValue(loMatch,"example");
					loMatch = null;
				}
				catch(Exception ex) 
				{ 
					string lcError = ex.Message; 
				}

				// *** Loop through the methods
				for (int lnMethods = 0; lnMethods < loObject.nMethodCount; lnMethods++) 
				{
					ObjectMethod loMethod = loObject.aMethods[lnMethods];

					if (loMethod.cSignature.IndexOf("#ctor") > 0) 
					{
						int x = 1;
					}

					if (loMethod.cRawParameters == "") 
						lcXPath = "/doc/members/member[@name='" +
							"M:" + loMethod.cSignature.Replace("()","") +  "']";  // .cNamespace + "." + loObject.cName + "." + loMethod.cName + "']";
					else
						lcXPath = "/doc/members/member[@name='" +
							"M:" + loMethod.cSignature + "']";

					//							"M:" + loObject.cNamespace + "." + loObject.cName + "." + loMethod.cName + 
					//							"(" + loMethod.cRawParameters + ")']";
					

					try 
					{
						loMatch = oDom.SelectSingleNode(lcXPath);

						loMethod.cHelpText = this.GetNodeValue(loMatch,"summary");
						loMethod.cSeeAlso = this.ExtractSeeAlso(ref loMethod.cHelpText);

						loMethod.cRemarks = this.GetNodeValue(loMatch,"remarks");
						loMethod.cExample = this.GetNodeValue(loMatch,"example");

						// *** Parse the parameters
						if (loMatch != null) 
						{
							string lcParmDescriptions = "";
							for (int x = 0; x < loMethod.nParameterCount; x++) 
							{
								lcParmDescriptions = lcParmDescriptions + 
									"<<b>>" + loMethod.aParameters[x] + "<</b>>\r\n"; 

								loSubMatch = loMatch.SelectSingleNode("param[@name='" + loMethod.aParameters[x]  +"']");
								if (loSubMatch != null) 
								{
									string lcValue = loSubMatch.InnerText;
									if (lcValue.Length > 0) 
									{
										lcValue = lcValue.TrimStart(new Char[3] {' ','\r','\n'}  );
										lcParmDescriptions = lcParmDescriptions + lcValue + "\r\n\r\n";
									}
									else 
									{
										lcParmDescriptions = lcParmDescriptions + "\r\n";
									}
								}

							}
							loMethod.cDescriptiveParameters = lcParmDescriptions;
						}
						
						loMatch = null;
					}
					catch(Exception ex) 
					{ 
						string lcError = ex.Message; 
					}
				}  // for methods

				// *** Loop through the properties
				for (int lnFields = 0; lnFields < loObject.nPropertyCount; lnFields++) 
				{
					ObjectProperty loProperty = loObject.aProperties[lnFields];
				
					string lcFieldPrefix = "F:";
					if (loProperty.cFieldOrProperty != "Field") 
						lcFieldPrefix = "P:";

					lcXPath = "/doc/members/member[@name='" + lcFieldPrefix +
						loObject.cNamespace + "." + loObject.cName + "." + loProperty.cName + "']";

					loMatch = oDom.SelectSingleNode(lcXPath);

					loProperty.cHelpText = this.GetNodeValue(loMatch,"summary");
					loProperty.cSeeAlso = this.ExtractSeeAlso(ref loProperty.cHelpText);

					loProperty.cRemarks = this.GetNodeValue(loMatch,"remarks");
					loProperty.cExample = this.GetNodeValue(loMatch,"example");
					loMatch = null;

				}  // for properties

				// *** Loop through the fields
				for (int lnFields = 0; lnFields < loObject.nEventCount; lnFields++) 
				{
					ObjectEvent loEvent = loObject.aEvents[lnFields];
				
					string lcFieldPrefix = "E:";

					lcXPath = "/doc/members/member[@name='" + lcFieldPrefix  +
						loObject.cNamespace + "." + loObject.cName + "." + loEvent.cName + "']";

					loMatch = oDom.SelectSingleNode(lcXPath);
					loEvent.cHelpText = this.GetNodeValue(loMatch,"summary");
					this.ExtractSeeAlso(ref loEvent.cHelpText);

					loEvent.cRemarks = this.GetNodeValue(loMatch,"remarks");
					loEvent.cExample = this.GetNodeValue(loMatch,"example");
					loMatch = null;

				}  // for events

			}  // for objects
            
			return true;
		} // ParseXMLProperties (method)

		/// <summary>	
		/// Retrieves the text from a child XML node
		/// </summary>
		/// <param name="Node"></param>
		/// <param name="Keyword"></param>
		/// <returns></returns>
		protected string GetNodeValue(XmlNode Node,string Keyword) 
		{
			XmlNode loSubMatch = null;
			string lcResult = "";

		

			if (Node != null) 
			{
				loSubMatch = Node.SelectSingleNode(Keyword);
				if (loSubMatch != null) 
				{
	
					if (Keyword == "remarks") 
					{
						int x = 10;
						x++;
					}

					// *** Need to pull inner XML so we can get the sub XML strings ilke <seealso> etc.
					lcResult = this.FixHelpString(loSubMatch.InnerXml);
				
					lcResult = this.FixupSeeLinks(lcResult);

					loSubMatch = null;

					// *** Deal with embedded list types - suboptimal at best but...
					if (lcResult.IndexOf("<list type=\"table\"") > -1 && lcResult.IndexOf("<description>") > -1 )
					{
						/// <list type="table">
						/// <item>
						///		<term>test item</term>
						///		<description>aDescription</description>
						/// </item>
						/// </list>
						string lcList = wwUtils.StrExtract(lcResult,"<list type=\"table\">","</list>");
 
						XmlDocument loDom = new XmlDocument();
						try 
						{
							loDom.LoadXml("<list>" + lcList + "</list>");
							StringBuilder sbHtml = new StringBuilder();

							sbHtml.Append("<RAWHTML>\r\n<table border=\"1\" cellpadding=\"3\" width=\"400\">\r\n");
							
							XmlNode Header = loDom.DocumentElement.SelectSingleNode("listheader");
							if (Header!= null) 
							{
								sbHtml.Append("<tr><th>" + Header.SelectSingleNode("term").InnerText);
								sbHtml.Append("</td><th>" + Header.SelectSingleNode("description").InnerText);
								sbHtml.Append("</th></tr>\r\n");
							}
							XmlNodeList nodeItems = loDom.DocumentElement.SelectNodes("item");
							foreach( XmlNode Item in nodeItems) 
							{
								sbHtml.Append("<tr><td>" + Item.SelectSingleNode("term").InnerText);
								sbHtml.Append("</td><td>" + Item.SelectSingleNode("description").InnerText);
								sbHtml.Append("</td></tr>\r\n");
							}
							sbHtml.Append("</table>\r\n</RAWHTML>\r\n");

							string lcOutput = sbHtml.ToString();
							int lnAt = lcResult.IndexOf(lcList);

							lcResult = wwUtils.StrTran( lcResult,"<list type=\"table\">" + lcList + "</list>",lcOutput,true);

						}
						catch(Exception e) { string lcError = e.Message;  }  /// ignore errors - leave as is
					}
				}
			}


			// *** Now un-markup what's left of the XML 
			lcResult = lcResult.Replace("&lt;","<");
			lcResult = lcResult.Replace("&gt;",">");

			return lcResult;
		}

		
		/// <summary>
		/// Fixes up <see></see> links in various formats.
		/// </summary>
		/// <param name="lcResult"></param>
		/// <returns></returns>
		protected string FixupSeeLinks(string lcResult) 
		{
			int Mode = 0;
			while (lcResult.IndexOf("<see") > -1) 
			{
				string lcCaption = "";
				string lcRef = "";

				// *** <see>Class Test</see>
				string lcSee = wwUtils.StrExtract(lcResult,"<see>","</see>",1);
				if (lcSee != string.Empty) 
				{
					// plain tag
					lcRef = lcSee;
					lcCaption = lcSee;
					Mode = 0;
				}
				else  
				{
					// *** Full CRef syntax: 
					lcSee = wwUtils.StrExtract(lcResult,"<see ","/>",1);
					if ( lcSee != string.Empty ) 
					{
						// <see cref="..." />
						Mode=2;
						lcRef = wwUtils.StrExtract(lcSee,"cref=\"","\"");
						if (lcSee != string.Empty) 
						{
							if (lcRef.IndexOf(":") == 1) 
								lcCaption = lcRef.Substring(2);
					
							else
								lcCaption = lcRef;
						}
					}
					else 
					{	
						lcSee = wwUtils.StrExtract(lcResult,"<see ","</see>",1);
						if (lcSee == string.Empty)
							break;

						Mode = 1;
						lcRef=wwUtils.StrExtract(lcSee,"cref=\"","\"");
						lcCaption = lcSee.Substring(lcSee.IndexOf(">")+1);
					}

					if (lcCaption == "") 
					{
						lcCaption = lcRef;
						if (lcCaption.IndexOf(":")==1) 
						  lcCaption = lcCaption.Substring(2);
						int At = lcCaption.IndexOf("(");
						if (At >-1)
							lcCaption = lcCaption.Substring(0,At);
					}
					if (lcRef == "") 
						lcRef = lcCaption; // try to find in Help File by name

#if VSNet_Addin
							// *** Check for .NET References -  point at MSDN
							if ( lcRef.StartsWith("System.") || lcRef.StartsWith("Microsoft.") )
								lcRef = "msdn:T:" + lcRef;
							else 
							{
								lcRef = "sig:" + lcRef;
							}
						
#else
					// *** Check for .NET References -  point at MSDN
					if ( lcRef.IndexOf("System.") == 2 || lcRef.IndexOf("Microsoft.") == 2)
						lcRef = "msdn:" + lcRef;
					else 
					{
						// *** Nope must be internal links - look up in project.
						// *** Strip of prefix and add sig: prefix
						if (lcRef.IndexOf(":") > -1) 
						{
							// *** Add method brackets on empty methods because that's
							// *** that's how we are importing them
							if (lcRef.StartsWith("M:") && lcRef.IndexOf("(") == -1)
								lcRef += "()";
 
							lcRef = "sig:" + lcRef.Substring(2);
						}
					}
#endif 
				}

	
				if (Mode == 0)
					lcResult = wwUtils.StrTran(lcResult,"<see>" + lcSee + "</see>",
						"<<%=TopicLink(\"" + lcCaption + "\",\"" + lcRef + "\") %>>" );
				else if (Mode == 1)
					lcResult = wwUtils.StrTran(lcResult,"<see " + lcSee + "</see>",
						"<<%=TopicLink(\"" + lcCaption + "\",\"" + lcRef + "\") %>>" );
				else if (Mode == 2)
					lcResult = wwUtils.StrTran(lcResult,"<see " + lcSee + "/>",
						"<<%=TopicLink(\"" + lcCaption + "\",\"" + lcRef + "\") %>>" );

			}

			return lcResult;
		}


		protected string GetAttributeValue(XmlNode Node,string Keyword,string Attribute) 
		{
			XmlNode loSubMatch = null;
			string lcResult = "";

			if (Node != null) 
			{
				loSubMatch = Node.SelectSingleNode(Keyword);
				if (loSubMatch != null) 
				{
					lcResult = this.FixHelpString(loSubMatch.InnerText);
					loSubMatch = null;
				}
			}

			return lcResult;
		}


		protected string GetHelpDetails() 
		{
			return "";
		}


		/// <summary>
		/// Fixes up a description string by splitting it
		/// into separate lines on Char(13)'s.
		/// </summary>
		/// <param name="lcString"></param>
		/// <returns></returns>
		protected string FixHelpString(string lcString) 
		{
			string[] lcStrings = lcString.Trim().Split((char) 13);
			lcString = "";

			int lnPad = -1;
			for (int x=0; x < lcStrings.Length; x++) 
			{
				if (lnPad == -1) 
				{
					int Len = lcStrings[x].TrimStart().Length;
					if (Len != lcStrings[x].Length)
						lnPad = lcStrings[x].Length - Len;
				}
				
				if (lnPad == -1)
					lcString += lcStrings[x].Trim('\n',' ') + "\r\n";
				else
				{
					if (lcStrings[x].Length > lnPad) 
						lcString += lcStrings[x].Substring(lnPad).Trim('\n') + "\r\n";
					else
						lcString += "\r\n";
				}
			}


			lcString = lcString.Replace("<para>","\r\n");
			lcString = lcString.Replace("</para>","\r\n");

			lcString = lcString.Replace("<pre>","<<pre>>");
			lcString = lcString.Replace("</pre>","<</pre>>");

			lcString = lcString.Replace("<c>","<<pre>>");
			lcString = lcString.Replace("</c>","<</pre>>");

			lcString = lcString.Replace("<code>","<<pre>>");
			lcString = lcString.Replace("</code>","<</pre>>");

			// *** Pick out <pre> blocks
			if (this.bWordWrap) 
			{
				ArrayList al = new ArrayList();
							
				while (true) 
				{
					string Text = wwUtils.StrExtract(lcString,"<<pre>>","<</pre>>");
					if (Text == null || Text == "") 
						break;

					lcString = lcString.Replace("<<pre>>" + Text + "<</pre>>","@@@@");
					al.Add(Text);
				}

				// *** Try to fix up line feeds - keep only double breaks
				lcString = lcString.Replace("\r\n\r\n","^^^^");
				lcString = lcString.Replace("\r\n"," ");
				lcString = lcString.Replace("^^^^","\r\n\r\n"); 

				foreach(string Text in al) 
				{
					lcString = wwUtils.StrTran(lcString,"@@@@","<<pre>>" + Text + "<</pre>>",true);
				}
			}
			
			return lcString;
		}

		/// <summary>
		/// Extracts seealso sections out of the XML into a CR delimited strings
		/// </summary>
		/// <param name="Body"></param>
		/// <returns></returns>
		protected string ExtractSeeAlso(ref string Body) 
		{
			if (Body.IndexOf("<seealso>") < 0)
				return "";

			string Result = "";
			string Extract = "XX";
			
			while(Extract != "") 
			{
				Extract = wwUtils.StrExtract(Body,"<seealso>","</seealso>");
				if (Extract != "") 
				{
					Result += Extract + "\r\n";
					Body = Body.Replace("<seealso>" + Extract + "</seealso>","");
				}
				else 
				{
					// *** Retrieve with CREF
					Extract = wwUtils.StrExtract(Body,"<seealso cref=","</seealso>");
					if(Extract != "") 
					{
						Result =  Extract.Substring(Extract.IndexOf(">")+1) + "\r\n";
						Body = Body.Replace("<seealso cref=" + Extract + "</seealso>",Result);
					}
				}
			}

			return Result;
		}
	}



}
