using System;
using Westwind.wwReflection.Tools;

namespace Westwind.wwHelp
{
	/// <summary>
	/// An implementation of the Help Builder IDE. This is the form based interface
	/// that you see when help builder runs. This class wrappers the Help Builder
	/// COM object (wwHelp.wwHelpForm) and makes passthrough calls via Reflection.
	/// </summary>
	public class wwHelpForm : wwHelpComWrapper
	{
		/// <summary>
		/// Overridden Constructor that must be passed a COM
		/// reference to the wwHelpForm Com object. All properties
		/// then map via Reflection.
		/// </summary>
		/// <param name="ComInstance">Instance of the Help Builder COM object (wwhelp.wwhelpForm)</param>
		public wwHelpForm(object ComInstance)
		{
			this.m_Inst = ComInstance;
		}

		/// <summary>
		/// Static Factory method that creates an instance of this class.
		/// Instantiates the COM object and creates the wrapper for you.
		/// </summary>
		/// <returns></returns>
		public static wwHelpForm CreateInstance() 
		{
			try 
			{
			   object HelpBuilder =  Activator.CreateInstance(Type.GetTypeFromProgID("wwHelp.wwHelpForm"));
    		   return new wwHelpForm(HelpBuilder);
			}
			catch
			{
			}
			return null;
		}

		/// <summary>
		/// Instance of the Help Builder Project (<see cref="wwHelp">wwHelp</see>) object. 
		/// </summary>
		public wwHelp Help
		{
			get 
			{ 
				return new wwHelp( wwUtils.GetPropertyCom(m_Inst,"oHelp") );
			}
		}

		/// <summary>
		/// Help Builder Configuration object that maps to the settings in wwHelp.ini.
		/// You can use the property interface to access values or the indexer on
		/// this object with the name of the property ( S = (string) Form.Config["LastIndex"] )
		/// </summary>
		public wwHelpConfig Config 
		{
			get 
			{
				return new wwHelpConfig( wwUtils.GetPropertyCom(m_Inst,"oConfig") );
			}
		}

		/// <summary>
		/// Flag that can be used to determine if a file is open in the IDE.
		/// </summary>
		public bool NoOpenFile
		{
			get { return (bool) wwUtils.GetPropertyCom(m_Inst,"lNoOpenFile"); }
			set { wwUtils.SetPropertyCom(m_Inst,"lNoOpenFile",value); }
		}

		/// <summary>
		/// Flag that can be used to determine if a topic is active.
		/// </summary>
		public bool NoOpenTopic
		{
			get { return (bool) wwUtils.GetPropertyCom(m_Inst,"lNoOpenTopic"); }
			set { wwUtils.SetPropertyCom(m_Inst,"lNoOpenTopic",value); }
		}



		/// <summary>
		/// Goes to the specified topic and displays it in the viewer in the current
		/// display mode.
		/// </summary>
		/// <param name="Pk"></param>
		/// <param name="TopicType"></param>
		/// <param name="NoSave"></param>
		/// <param name="JumpToAnchor"></param>
		public void GoTopic(string Pk,TopicTypes TopicType,bool NoSave,string JumpToAnchor)
		{
			wwUtils.CallMethodCom(this.m_Inst,"GoTopic",Pk,(int) TopicType,NoSave,JumpToAnchor);
		}

		/// <summary>
		/// Goes to the specified topic and displays it in the viewer in the current
		/// display mode.
		/// </summary>
		public void GoTopic() 
		{
			this.GoTopic("",TopicTypes.Pk,false,"");
		}

		/// <summary>
		/// Goes to the specified topic and displays it in the viewer in the current
		/// display mode.
		/// </summary>
		public void GoTopic(string Pk) 
		{
			this.GoTopic(Pk,TopicTypes.Pk,false,"");
		}


		/// <summary>
		/// Runs one of the built in Wizards.
		/// 
		/// wizNewTopic
		/// wizImport
		/// wizBuildOutput
		/// </summary>
		/// <param name="WizardName"></param>
		public void RunWizard(string WizardName) 
		{
			wwUtils.CallMethodCom(this.m_Inst,"RunWizard",WizardName,"");
		}

		/// <summary>
		/// Opens a project file in the IDE.
		/// </summary>
		/// <param name="Filename">Full path to the HBP project file to open</param>
		/// <param name="Force">Force to re-open a file if already open. Otherwise open file is not reopened</param>
		/// <param name="Silent">If true no dialogs are shown if file doesn't exist. Fails silently</param>
		/// <returns></returns>
		public bool Open(string Filename, bool Force, bool Silent) 
		{
				return (bool) wwUtils.CallMethodCom(this.m_Inst,"Open",Filename,Force,Silent);
		}

		/// <summary>
		/// Opens a project file in the IDE.
		/// </summary>
		/// <param name="Filename">Full path to the HBP project file to open.</param>
		/// <returns></returns>
		public bool Open(string Filename) 
		{
			return this.Open(Filename,false,false);
		}

		/// <summary>
		/// Closes the currently open IDE Project
		/// </summary>
		/// <returns></returns>
		public void CloseProject() 
		{
			wwUtils.CallMethodCom(this.m_Inst,"CloseProject");
		}


		/// <summary>
		/// Saves the currently active topic and refreshes the UI.
		/// </summary>
		/// <returns></returns>
		public void SaveTopic() 
		{
			wwUtils.CallMethodCom(this.m_Inst,"SaveTopic",false);
		}

		/// <summary>
		/// Deletes the currently active IDE topic. Prompts with dialog.
		/// </summary>
		/// <returns></returns>
		public void DeleteTopic()
		{
			wwUtils.CallMethodCom(this.m_Inst,"DeleteTopic");
		}

#region Display Methods
		
		public ViewerModes ViewerMode
		{
			get 
			{ 
				int Val =  Convert.ToInt32(wwUtils.GetPropertyCom(m_Inst,"nViewMode")); 
				return (ViewerModes) Val;
			}
			set { this.AddViewer(ViewerMode); }
		}

		/// <summary>
		/// Switches View modes for the Help Builder Form.
		/// </summary>
		/// <param name="ViewerMode"></param>
		public void AddViewer(ViewerModes ViewerMode) 
		{
			wwUtils.CallMethodCom(this.m_Inst,"AddViewer",(int) ViewerMode);
		}

		/// <summary>
		/// 
		/// </summary>
		public StatusTypes StatusType
		{
			get { return (StatusTypes) wwUtils.GetPropertyCom(m_Inst,"nStatusType"); }
			set { wwUtils.SetPropertyCom(m_Inst,"nStatusType",value); }
		}

		/// <summary>
		/// Displays a message on the statusbar. This version allows specifying
		/// which panel the message displays in and optionally allows progress
		/// report information. 
		/// </summary>
		/// <param name="Message"></param>
		public void StatusMessage(string Message,string Panel,int Percent,string Header) 
		{
			wwUtils.CallMethodCom(this.m_Inst,"StatusMessage",Message,Panel,Percent,Header);
		}
		
		/// <summary>
		/// Displays a message on the statusbar
		/// </summary>
		public void StatusMessage(string Message) 
		{
			this.StatusMessage(Message,"",0,"");
		}

		/// <summary>
		/// If true makes the Help Builder Window topmost and keeps it there.
		/// </summary>
		public bool AlwaysOntop
		{
			get { return (bool) wwUtils.GetPropertyCom(m_Inst,"AlwaysOntop"); }
			set { wwUtils.SetPropertyCom(m_Inst,"AlwaysOntop",value); }
		}

		/// <summary>
		/// Set/Returns the Caption of the Help Builder Environment
		/// </summary>
		public string Caption
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"Caption"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Caption",value); }
		}

		/// <summary>
		/// Activates the Body field on the form for the current topic. Does not work
		/// if in HTML mode - call AddViewer(1) to force into HTML view first.
		/// </summary>
		public void ActivateBodyField() 
		{
			wwUtils.CallMethodCom(this.m_Inst,"ActivateBodyField");
		}

		/// <summary>
		/// Refreshes the toolbar with common settings for the current mode.
		/// </summary>
		public void SetToolBar() 
		{
				wwUtils.CallMethodCom(this.m_Inst,"SetToolbar");
		}

		/// <summary>
		/// Activates the Help Builder Window and makes it the Active application window.
		/// </summary>
		public void Activate() 
		{
			wwUtils.CallMethodCom(this.m_Inst,"Activate");
		}

		/// <summary>
		/// Display the form.
		/// </summary>
		public void Show() 
		{
			wwUtils.CallMethodCom(this.m_Inst,"Show");
		}

		public void Close()
		{
			wwUtils.CallMethodCom(this.m_Inst,"Close");
		}

		public int Top
		{
			get { return Convert.ToInt32( wwUtils.GetPropertyCom(m_Inst,"Top")); }
			set { wwUtils.SetPropertyCom(m_Inst,"Top",value); }
		}

		public int Left
		{
			get { return Convert.ToInt32( wwUtils.GetPropertyCom(m_Inst,"Left") ); }
			set { wwUtils.SetPropertyCom(m_Inst,"Left",value); }
		}
		public int Width
		{
			get { return Convert.ToInt32( wwUtils.GetPropertyCom(m_Inst,"Width")); }
			set { wwUtils.SetPropertyCom(m_Inst,"Width",value); }
		}
		public int Height
		{
			get { return Convert.ToInt32(wwUtils.GetPropertyCom(m_Inst,"Height")); }
			set { wwUtils.SetPropertyCom(m_Inst,"Height",value); }
		}
#endregion


	}
}
