using System;
using Westwind.wwReflection.Tools;

namespace Westwind.wwHelp
{

	/// <summary>
	/// The Topic class provides information about an individual Help Builder
	/// topic. It maps essentially to the underlying fields in the database.
	/// 
	/// This class also supports an indexer for custom fields so you can
	/// reference any fields that exist in the table but are not exposed
	/// as properties via the generic indexer:
	///  
	/// S = (string) Help.Topic["MyCustomField"];
	/// </summary>
	public class wwHelpTopic : wwHelpComWrapper
	{
		/// <summary>
		/// Overridden Constructor that must be passed a COM
		/// reference to the wwHelp Com object. All properties
		/// then map via Reflection.
		/// </summary>
		/// <param name="ComInstance">Instance of the Help Builder COM object (wwhelp.wwhelp)</param>
		public wwHelpTopic(object ComInstance)
		{
			m_Inst = ComInstance;
		}
	
		/// <summary>
		/// Topic indexer allows you to directly retrieve a field by
		/// its name. Can be used for any custom fields you add to the
		/// table on your own.
		/// </summary>
		public object this[string index] 
		{
			get { return wwUtils.GetPropertyCom(this,index); }
			set { wwUtils.SetPropertyCom(this,index,value); }
		}

		/// <summary>
		/// The primary key for the topic. Primary identifier.
		/// </summary>
		public string Pk
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"Pk"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Pk",value); }
		}

		/// <summary>
		/// The parent topics Pk. Used manage self referential tree display.
		/// </summary>
		public string ParentPk
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"ParentPk"); }
			set { wwUtils.SetPropertyCom(m_Inst,"ParentPk",value); }
		}

		/// <summary>
		/// The title of the topic
		/// </summary>
		public string Topic
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"Topic"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Topic",value); }
		}

		/// <summary>
		/// The body content of the topic
		/// </summary>
		public string Body
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"Body"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Body",value); }
		}


		/// <summary>
		/// The Topic Type: TOPIC/HEADER/CLASSHEADER/CLASSMETHOD etc.
		/// </summary>
		public string Type
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"Type"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Type",value); }
		}

		
		/// <summary>
		/// Determines whether the content of this topic points to an external link.
		/// </summary>
		public bool External
		{
			get { return (bool) wwUtils.GetPropertyCom(m_Inst,"External"); }
			set { wwUtils.SetPropertyCom(m_Inst,"External",value); }
		}

		/// <summary>
		/// Name of the Class that this topic belongs to. Used only for CLASS... types.
		/// </summary>
		public string Class
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"Class"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Class",value); }
		}

		/// <summary>
		/// The name of the class member method/property/event/field
		/// </summary>
		public string Member
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"Method"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Method",value); }
		}

		/// <summary>
		/// The Scope of a class member: Public/Private/Protected
		/// </summary>
		public string Scope
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"Scope"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Scope",value); }
		}

		/// <summary>
		/// The prototype sytnax for the topic
		/// </summary>
		public string Syntax
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"Syntax"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Syntax",value); }
		}

		/// <summary>
		/// Any parameters that are passed. This content is fully formatted as an HTML string.
		/// </summary>
		public string Parameters
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"Parameters"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Parameters",value); }
		}

		/// <summary>
		/// Describes the return value for a function
		/// </summary>
		public string Returns
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"Returns"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Returns",value); }
		}

		/// <summary>
		/// Any special notes for this topic.
		/// </summary>
		public string Remarks
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"Remarks"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Remarks",value); }
		}

		/// <summary>
		/// An example code snippet. The text in this snippet is automatically formmated
		/// as in a PRE tag.
		/// </summary>
		public string Example
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"Example"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Example",value); }
		}

		/// <summary>
		/// Related Links. The content in this topic is formatted as individual lines
		/// Use wwHelp.InsertSeeAlsoTopics() to convert this field into HTML links.
		/// </summary>
		public string SeeAlso
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"SeeAlso"); }
			set { wwUtils.SetPropertyCom(m_Inst,"SeeAlso",value); }
		}

		/// <summary>
		/// Additional Search Keywords for this topic for the index
		/// </summary>
		public string Keywords
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"Keywords"); }
			set { wwUtils.SetPropertyCom(m_Inst,"KeyWords",value); }
		}
		
		/// <summary>
		/// Help Id/Context ID used in older applications
		/// </summary>
		public int HelpId
		{
			get { return (int) Convert.ToInt32( wwUtils.GetPropertyCom(m_Inst,"HelpId")) ; }
			set { wwUtils.SetPropertyCom(m_Inst,"HelpI", Convert.ToDouble(value)); }
		}

		/// <summary>
		/// The sort order for this topic. 0 means default order which is
		/// by type and then alpha numeric. Higher numbers override the default
		/// and sort to the top.
		/// </summary>
		public int SortOrder
		{
			get { return (int) Convert.ToInt32( wwUtils.GetPropertyCom(m_Inst,"SortOrder") ); }
			set { wwUtils.SetPropertyCom(m_Inst,"SortOrder", Convert.ToDouble( value)); }
		}

		/// <summary>
		/// Timestamp for the topic
		/// </summary>
		public DateTime Updated
		{
			get { return (DateTime) wwUtils.GetPropertyCom(m_Inst,"Updated"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Updated",value); }
		}

		/// <summary>
		/// Determines whether follow up is required
		/// </summary>
		public bool FollowUp
		{
			get { return (bool) wwUtils.GetPropertyCom(m_Inst,"FollowUp"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Followup",value); }
		}
		
		/// <summary>
		/// .NET namespace.member signature for the member.
		/// </summary>
		public string Signature
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"Signature"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Signature",value); }
		}


		/// <summary>
		/// Name of an .NET assembly for .NET class topics.
		/// </summary>
		public string Assembly
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"Assembly"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Assembly",value); }
		}

		/// <summary>
		/// Name of the namespace this class lives in.
		/// </summary>
		public string Namespace
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"Namespace"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Namespace",value); }
		}

		/// <summary>
		/// The immediate parent class for this class
		/// </summary>
		public string Inherits
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"Inherits"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Inherits",value); }
		}

		/// <summary>
		/// An Html based list of the Inheritance structure.
		/// </summary>
		public string Inh_Tree
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"Inh_Tree"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Inh_tree",value); }
		}

		/// <summary>
		/// The interfaces this class implements. 
		/// </summary>
		public string Implements
		{
			get { return (string) wwUtils.GetPropertyCom(m_Inst,"Implements"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Implements",value); }
		}

		/// <summary>
		/// returns whether this class member is static
		/// </summary>
		public bool Static
		{
			get { return (bool) wwUtils.GetPropertyCom(m_Inst,"Static"); }
			set { wwUtils.SetPropertyCom(m_Inst,"Static",value); }
		}

		
	}
}
